RPM_DEB_INSTALLER_HOME="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)"
SCRIPT_HOME="$(cd ${RPM_DEB_INSTALLER_HOME}/bin && pwd)"
RPM_DEB_THIRDPARTY_HOME="${RPM_DEB_INSTALLER_HOME}/third-party"
JF_PRODUCT_HOME="/opt/jfrog/${PRODUCT_NAME}"
INSTALLER_STATE_FILE="${JF_PRODUCT_HOME}/var/etc/installerState.yaml"
YQ_BINARY="${RPM_DEB_THIRDPARTY_HOME}/yq/yq"
APP_DEFAULT_FILE="${JF_PRODUCT_HOME}/app/bin/${RPM_DEB_PRODUCT_PACK_NAME}.default"
RPM_DEB_PRODUCT_SERVICE_NAME="${RPM_DEB_PRODUCT_PACK_NAME}"
LINUX_PRODUCT_SERVICE_NAME="${RPM_DEB_PRODUCT_PACK_NAME}"
PACK_LIST=${RPM_DEB_PRODUCT_PACK_NAME}
AVAILABLETOOLS="${RPM_DEB_PRODUCT_PACK_NAME}"
LIST_TEST=""
CLEAN_DIR=""

. ${SCRIPT_HOME}/systemYamlHelper.sh
. ${SCRIPT_HOME}/rpmDebHelper.sh

# Get Users Input from terminal
getUserInput(){
    local choice="na"
    local text_to_display=$1
    local default_choice=$2
    users_choice=
    : ${read_timeout:=.5}
    if [ ! -z "$default_choice" ]; then
        text_to_display="$1 [$default_choice]:"
    else
        text_to_display="$1 :"
    fi
    sleep $read_timeout #This ensures correct placement of the question.
    read -p "$text_to_display" choice
    if [ ! -z "$default_choice" ]; then
        : ${choice:=$default_choice}
    fi
    users_choice=$choice
    sleep $read_timeout #This ensures correct logging
}
# Validate available set of packages for the specific product
rpmDeb_checkPack() {
    local USER_END_INSTALLS="$1"
    AVAILABLETOOLS=''
    for appName in $(echo ${USER_END_INSTALLS[@]})
    do
       package_isInstalled "${appName}"
       if [ $? -eq 0 ] ; then
          AVAILABLETOOLS="${AVAILABLETOOLS} ${PACKAGE_LIST}"
       fi
    done
}
# Check whether a package exist
package_isInstalled(){
    local packName=$1
    local name=''
    local version=''
    PACKAGE_LIST=''
    arrIN=(${packName//=/ })
    packName=${arrIN[0]}
    if [ -z ${packName} ] || [ "${packName}" == "" ]
    then
        errorExit "isInstalled: missing packName value"
    fi
    case ${LINUX_DISTRIBUTION} in
        CentOS|RedHat|SLES|AmazonLinux)
            PACKAGE_LIST=$(rpm -qa --nosignature --queryformat '%{NAME}\n' ${packName} | tr '\r\n' ' ' )
            name="$(rpm -qa --nosignature --queryformat '%{NAME}\n' ${packName} 2>/dev/null)"
            version="$(rpm -qa --nosignature --queryformat '%{VERSION}\n' ${packName} 2>/dev/null)"
        ;;
        Debian|Ubuntu)
	    PACKAGE_LIST=$(dpkg-query -l ${packName} | grep 'ii' | awk -F' ' '{print $2}' | tr '\r\n' ' ' )
            name=$(dpkg-query -W -f='${binary:Package}' ${packName} 2>/dev/null)
            version=$(dpkg-query -W -f='${Version}' ${packName} 2>/dev/null)
        ;;
        *)
            errorExit "isInstalled: Distribution ${LINUX_DISTRIBUTION} is not supported"
            return 1
        ;;
    esac
    # if name and version is not available consider it as not installed and return 1
    # let the caller decide what to do based on this response
    if [ -z "${name}" ] || [ "${name}" == "" ] || [ -z "${version}" ] || [ "${version}" == "" ]
    then
        return 1
    else
        return 0
    fi
}

appendDotToKeyPath(){
    local key=$1
    if [[ ${key:0:1} == "." ]] ; then 
      echo $key; 
    else 
      echo "."$key; 
    fi
}

# check installerState.yaml file for installed Third party packages
check_stateFile()  {
    yaml_value=$1
    package_name=$2
    add_dir=$3
    yaml_value=$(appendDotToKeyPath "$yaml_value")
    # TODO (shahin) Add a method in systemYaml helper and use it from there.
    INSTALL_STATE=$("${YQ_BINARY}" e "${yaml_value}" "${INSTALLER_STATE_FILE}")
    if [[ "$INSTALL_STATE" == "y" ]]
    then
        THIRD_PARTY_PACKS="${THIRD_PARTY_PACKS} ${package_name}"
        THIRD_PARTY_DATA_DIR="${THIRD_PARTY_DATA_DIR} ${add_dir}"
    fi
}
# Delete a list of folders
check_deleteDir() {
    delete_dir=$1
    local dirList=($delete_dir)
    for dirName in "${dirList[@]}"
    do
        logger "Removing $dirName Directory"
        if [[ -d $dirName ]]; then
            rm -rf $dirName
        fi
    done
}
# Main function
main() {
    validateSudo
    io_getLinuxDistribution
    if [[ ! -d ${JF_PRODUCT_HOME} ]]; then
        errorExit "JF_PRODUCT_HOME does not exist"
    fi
    DEFAULT_FILE_APP_DIR=$(cat ${APP_DEFAULT_FILE} | grep -o '^[^#]*' | grep JF_PRODUCT_VAR | awk -F'JF_PRODUCT_VAR=' '{print $2}')
    APP_DIR=${DEFAULT_FILE_APP_DIR:-"/var/opt/jfrog/$PRODUCT_NAME"}
    PRODUCT_DIRS="/opt/jfrog/$PRODUCT_NAME/var $APP_DIR"
    #getUserInput "Do you want to uninstall Third Party Applications [y/N]" "N"
    UNINSTALL_THIRD_PARTY="n"
    getUserInput "Do you want to remove Data Directories [y/N]" "N"
    CLEAN_DATA_DIR=$users_choice
    if [[ "${UNINSTALL_THIRD_PARTY}" == "Y" || "${UNINSTALL_THIRD_PARTY}" == "y" ]];
    then
        check_stateFile "installer.install_postgresql" "${PG_PACK}" "${PG_DIRS}"
        PACK_LIST="${PACK_LIST} ${THIRD_PARTY_PACKS}"
        rpmDeb_checkPack "${PACK_LIST}"
    fi
    if [[ "${CLEAN_DATA_DIR}" == "Y" || "${CLEAN_DATA_DIR}" == "y" ]];
    then
        CLEAN_DIR="$APP_DIR"
        if [[ "${UNINSTALL_THIRD_PARTY}" == "Y" || "${UNINSTALL_THIRD_PARTY}" == "y" ]];
        then
            THIRD_PARTY_DATA_DIR="${THIRD_PARTY_DATA_DIR} ${CONFIRMED_THIRD_PARTY_DIRS}"
            CLEAN_DIR="${CLEAN_DIR} ${THIRD_PARTY_DATA_DIR}"
        fi
    fi
    bannerSubSection "Uninstallation of the application will be removing the following"
    if [ -n "${AVAILABLETOOLS}" ]
    then
        printf "Applications : \n\n"
        echo ${AVAILABLETOOLS} | tr ' ' '\n'
    fi
    if [ -n "${CLEAN_DIR}" ]
    then
        printf "\nDirectories : \n\n"
        echo ${CLEAN_DIR} | tr ' ' '\n'
    fi
    bannerImportant "\t\tRemoving the above Packages/Directories are Non recoverable"
    getUserInput "Do you want to continue with Uninstallation [y/N]" "N"
    CONFIRM_UNINSTALL=$users_choice
    if [[ "${CONFIRM_UNINSTALL}" == "Y" || "${CONFIRM_UNINSTALL}" == "y" ]];
    then
	    echo "Proceeding with Uninstallation"
    else
        echo "Exiting Uninstallation"
        exit 1
    fi
    rpmDeb_invokeSystemUnInstaller "${AVAILABLETOOLS}"
    check_deleteDir "${CLEAN_DIR}"
}

main