
linux_init(){
    JF_PRODUCT_HOME="$(cd "$(dirname "${BASH_SOURCE[0]}")"/../.. && pwd)"

    rpmDeb_init

    RPM_DEB_INSTALLER_HOME=${JF_PRODUCT_HOME}
    SCRIPT_HOME="$(cd ${JF_PRODUCT_HOME}/app/bin && pwd)"
    RPM_DEB_THIRDPARTY_HOME="${JF_PRODUCT_HOME}/app/third-party"
    LINUX_THIRDPARTY_HOME="${JF_PRODUCT_HOME}/app/third-party"
    INSTALLATION_LOG_FILE="${JF_PRODUCT_HOME}/var/log/install.log"

    JF_PRODUCT_VAR_DEFAULT=${JF_PRODUCT_HOME}/var

    INSTALLER_YAML=${SCRIPT_HOME}/${FILE_NAME_INSTALLER_YAML}
    RPM_DEB_DATA_DIR=${JF_PRODUCT_HOME}/var

    PRODUCT_USER=${PRODUCT_USER:-"${DEFAULT_PRODUCT_USER}"}
    PRODUCT_GROUP=${PRODUCT_GROUP:-"${PRODUCT_USER}"}

    linux_common_init
}

linux_createLogFile(){
    redirectLogsToFile #creates variable DESTINATION_LOG_FILE
    INSTALLATION_LOG_FILE="${JF_PRODUCT_HOME}/var/log/install.log"
    ETC_FOLDER="${JF_PRODUCT_HOME}/var/etc"
    createDir "$(dirname ${INSTALLATION_LOG_FILE})" "${PRODUCT_USER}" "${PRODUCT_GROUP}"
    createDir "$ETC_FOLDER" "${PRODUCT_USER}" "${PRODUCT_GROUP}"
    configureLogOutput
}

linux_productStartInfo(){
    local serviceName="$1"
    local productName="${2:-product}"

    if [[ -z "${productName}" ]]; then
        return 0
    fi

    cat << END_START_USAGE

NOTE : Make sure ${JF_PRODUCT_HOME} is owned by ${PRODUCT_USER}.
    chown -R ${PRODUCT_USER}:${PRODUCT_GROUP} ${JF_PRODUCT_HOME}

To start ${productName}:
(1) login as ${PRODUCT_USER} user
(2) ${JF_PRODUCT_HOME}/app/bin/${serviceName}.sh start

To check ${productName} status:
(1) login as ${PRODUCT_USER} user
(2) ${JF_PRODUCT_HOME}/app/bin/${serviceName}.sh status

Installation directory was set to ${JF_PRODUCT_HOME}
You can find more information in the log directory ${JF_PRODUCT_HOME}/var/log

System configuration templates can be found under ${JF_PRODUCT_HOME}/var/etc
Copy any configuration you want to modify from the template to ${JF_PRODUCT_HOME}/var/etc/system.yaml

END_START_USAGE
}

linux_endInfo() {
    echo ""
    logger "Installation complete. Installation log can be found at [${INSTALLATION_LOG_FILE}]"

    linux_productStartInfo "${LINUX_PRODUCT_SERVICE_NAME}" "${PRODUCT_NAME}"

    getJFrogURLIfAvailable >/dev/null || true

    echo ""
    if [ ! -z "${JF_SHARED_JFROGURL}" ]; then
        logger "Once the application has started, it can be accessed at [${JF_SHARED_JFROGURL}]"
    fi
    echo ""
}

linux_hook_start_getUserInputs(){
    logSilly "Method ${FUNCNAME[0]}"
}

wrapper_hook_start_getUserInputs(){
    linux_hook_start_getUserInputs
}

linux_checkPrerequisitesSoftware() {
    # Validate that any pre-requisites specified by the 3rd party libraries are present
    local userEndInstalls="$1"
    local depencenciesURL="$2"
    local packDisplayName=""
    local packName=""
    if [[ -z "${userEndInstalls}" || -z "${depencenciesURL}" ]]; then
        return 0
    fi

    local unavailableTools=""
    local toolList=($userEndInstalls)

    for packKeyValue in "${toolList[@]}"
    do
        packName=$(echo "${packKeyValue}" | awk -F"=" '{print $1}')
        packDisplayName=$(echo "${packKeyValue}" | awk -F"=" '{print $2}')

        [ -z "${packDisplayName}" ] && packDisplayName="${packName}" || true

        if [[ "$(io_commandExists ${packName})" != "yes" ]]; then
          unavailableTools="${unavailableTools} ${packDisplayName}"
        fi
    done

    if [ ! -z "${unavailableTools}" ]; then
        logError "Some dependencies [ ${unavailableTools} ] the product needs are missing"
        errorExit "Please check Prerequisites section under ${depencenciesURL} on instructions to install them and then try again"
    fi
}

rpmDeb_thirdPartyInfoToYaml(){
    rpmDeb_PostgresInfoToYaml "${FLAG_Y}"
}

linux_preUserInputChecks(){
    wrapper_setSystemValue "$KEY_POSTGRES_INSTALL" "$FLAG_N" "${overwrite}"
    SKIP_POSTGRES_SETUP=${FLAG_Y}
}

linux_askPostgresdetails(){
    getYamlValue "$KEY_POSTGRES_INSTALL" "$INSTALLER_YAML" "false"
    isPostgresExternalised=$([ "$YAML_VALUE" == "$FLAG_Y" ] && echo -n "$FLAG_N" || echo -n "$FLAG_Y")

    if [ "$isPostgresExternalised" == "$FLAG_Y" ]; then
        _askExternalDBInputs
    fi
}

linux_setOwnership(){
    for item in ${JF_SYSTEM_YAML} ${INSTALLER_STATE_YAML_FILE} ${INSTALLATION_LOG_FILE} ; do
        io_setOwnership "${item}" "${PRODUCT_USER}" "${PRODUCT_GROUP}"
    done
}

linux_usage () {
    echo "Usage: ${0}"
    cat << END_USAGE

-h | --help                                       : [optional] display usage
-u | --user                                       : [optional] user which will be used to run the product, it will be created if its unavailable
-g | --group                                      : [optional] group which will be used to run the product, it will be created if its unavailable
--pg-data                                         : [optional] (default: /usr/local/pgsql/data) pass the data directory for postgres installation 
END_USAGE
    sleep .5
}

linux_processOptions() {
    VERBOSE_MODE=false
                
    while [[ $# > 0 ]]; do
        case "$1" in
            -h | --help)
                linux_usage
                exit 0
            ;;
            -u | --user)
                PRODUCT_USER="$2"
                shift 2
            ;;
            -g | --group)
                PRODUCT_GROUP="$2"
                shift 2
            ;;
            --pg-data)
                LINUX_POSTGRES_DATA="$2"
                shift 2
            ;;
            *)
                linux_usage
                exit 1
            ;;
        esac
    done
}

linux_installService_usage () {
    echo "Usage: ${0}"
    cat << END_USAGE

-h | --help                                       : [optional] display usage
-u | --user                                       : [optional] user which will be used to run the product, it will be created if its unavailable
-g | --group                                      : [optional] group which will be used to run the product, it will be created if its unavailable
END_USAGE
    sleep .5
}

linux_installService_processOptions() {
    VERBOSE_MODE=false
                
    while [[ $# > 0 ]]; do
        case "$1" in
            -h | --help)
                linux_installService_usage
                exit 0
            ;;
            -u | --user)
                PRODUCT_USER="$2"
                shift 2
            ;;
            -g | --group)
                PRODUCT_GROUP="$2"
                shift 2
            ;;
            *)
                linux_installService_usage
                exit 1
            ;;
        esac
    done
}

linux_setProductHome() {
    local defaultEnv=$1

    if [[ -z "${JF_PRODUCT_HOME}" ]] || [[ -z "${defaultEnv}" ]]; then
        return 0;
    fi

    addOrReplaceProperty "export JF_PRODUCT_HOME" "${JF_PRODUCT_HOME}" "${defaultEnv}" \
        || errorExit "Failed to set JF_PRODUCT_HOME in ${defaultEnv}"
}

linux_createRunDir() {
    local runDir="$JF_PRODUCT_HOME/app/run"
    if [ ! -d "${runDir}" ]; then
        mkdir -p "${runDir}" || errorExit "Failed to create ${runDir}"
    fi
}

linux_stopService() {
    local serviceName="$1"
    [ ! -z "${serviceName}" ] || return 0
    local initFile="/etc/init.d/${serviceName}"

    # Shutting down the product service if running
    if [ -e "${initFile}" ]; then
        logger "Stopping the ${serviceName} service"
        ${initFile} stop >/dev/null 2>&1
        [ -x /bin/systemctl ] && /bin/systemctl stop "${LINUX_SERVICE_NAME}" >/dev/null 2>&1 || true
    fi
}

linux_installService() {
    linux_uninstallService >/dev/null
    if [[ "${IS_SYSTEMD_ENABLED}" != "${FLAG_Y}" ]]; then
        linux_installInitdService
        return;
    fi

    if systemctl -h > /dev/null 2>&1; then
        # Installing service from type systemd
        linux_installSystemdService
    else
        # Installing service from type init.d
        linux_installInitdService
    fi
}

linux_uninstallService() {
    linux_stopService "${LINUX_SERVICE_NAME}"
    linux_uninstallInitdService
}

linux_uninstallInitdService(){
    logger "Removing $LINUX_SERVICE_NAME service from auto-start"
    local removeOk=
    if [ -x /bin/systemctl ]; then
        /bin/systemctl disable "${LINUX_SERVICE_NAME}" >/dev/null 2>&1 && removeOk=true
        local SUSE=$(cat /etc/issue | grep -io suse)
        local NOSUSE=$(echo $SUSE | tr "[:upper:]" "[:lower:]")
        local UNAME=$(uname | tr "[:upper:]" "[:lower:]")    
        local unitFile=

        if [ "$UNAME" == "linux" ] && [ "$NOSUSE" != "suse" ]; then
            unitFile="/lib/systemd/system/${LINUX_SERVICE_NAME}.service"
        else
            unitFile="/usr/lib/systemd/system/${LINUX_SERVICE_NAME}.service"
        fi
        rm -f "$unitFile" >/dev/null 2>&1;
        /bin/systemctl daemon-reload >/dev/null 2>&1;
        /bin/systemctl reset-failed "${LINUX_SERVICE_NAME}" >/dev/null 2>&1;
    fi

    if [ -x /usr/sbin/update-rc.d ]; then
        /usr/sbin/update-rc.d -f "${LINUX_SERVICE_NAME}" remove >/dev/null 2>&1 && removeOk=true
    elif [ -x /usr/sbin/chkconfig ]; then
        /usr/sbin/chkconfig --del "${LINUX_SERVICE_NAME}" >/dev/null 2>&1 && removeOk=true
    elif [ -x /sbin/chkconfig ]; then
        /sbin/chkconfig --del "${LINUX_SERVICE_NAME}" >/dev/null 2>&1 && removeOk=true
    fi

    [ ${removeOk} ] || warn "Uninstalling service ${LINUX_SERVICE_NAME} failed"

    removeOk=
    if [ -e "${LINUX_INIT_SERVICE_FILE}" ]; then
        rm -f ${LINUX_INIT_SERVICE_FILE} >/dev/null 2>&1 && removeOk=true
        [ -e ${LINUX_INIT_SERVICE_FILE}.init.backup ] && rm -f ${LINUX_INIT_SERVICE_FILE}.init.backup >/dev/null 2>&1 || true
    else
        removeOk=true
    fi

    [ ${removeOk} ] || warn "Removing $LINUX_INIT_SERVICE_FILE failed, can be removed manually"
}

linux_installSystemdService() {
    SYSTEMD_UNIT_TEMPLATE=${JF_PRODUCT_HOME}/app/misc/service/jf-systemd.service

    SUSE=$(cat /etc/issue | grep -io suse)
    NOSUSE=$(echo $SUSE | tr "[:upper:]" "[:lower:]")
    UNAME=$(uname | tr "[:upper:]" "[:lower:]")
    if [ "$UNAME" == "linux" ] && [ "$NOSUSE" != "suse" ]; then
        SYSTEMD_UNIT_FILE="/lib/systemd/system/${LINUX_SERVICE_NAME}.service"
    else
        SYSTEMD_UNIT_FILE="/usr/lib/systemd/system/${LINUX_SERVICE_NAME}.service"
    fi

    local initFile=/etc/init.d/${LINUX_SERVICE_NAME}
    if [ -e ${initFile} ]; then
        mv ${initFile} ${initFile}.disabled
    fi

    cp -f ${SYSTEMD_UNIT_TEMPLATE} ${SYSTEMD_UNIT_FILE}
    chmod 644 ${SYSTEMD_UNIT_FILE}

    sed -i -e "s,__JF_PRODUCT_HOME__,$JF_PRODUCT_HOME,g" ${SYSTEMD_UNIT_FILE}

    # Use the systemctl command to enable the service
    echo "Initializing $LINUX_SERVICE_NAME service with systemctl..."
    systemctl daemon-reload &>/dev/null
    systemctl enable ${LINUX_SERVICE_NAME} &>/dev/null
    systemctl list-unit-files --type=service | grep ${LINUX_SERVICE_NAME}.service &>/dev/null || errorExit "Could not install ${LINUX_SERVICE_NAME} service"
}

linux_installInitdService() {
    [ ! -z "${LINUX_INIT_SERVICE_FILE}" ]     || errorExit "LINUX_INIT_SERVICE_FILE is not set, could not install service"
    [ ! -z "${LINUX_INIT_SERVICE_TEMPLATE}" ] || errorExit "LINUX_INIT_SERVICE_TEMPLATE is not set, could not install service"
    [ ! -z "${LINUX_SERVICE_NAME}" ]          || errorExit "LINUX_SERVICE_NAME is not set, could not install service"
    [ ! -z "${JF_PRODUCT_HOME}" ]             || errorExit "JF_PRODUCT_HOME is not set, could not install service"

    local chkconfigOK=

    if [ -e "$LINUX_INIT_SERVICE_FILE" ]; then
        cp -f ${LINUX_INIT_SERVICE_FILE} ${LINUX_INIT_SERVICE_FILE}.init.backup >/dev/null 2>&1
    fi

    logger "Installing ${LINUX_SERVICE_NAME} as a service"
    
    cp -f ${LINUX_INIT_SERVICE_TEMPLATE} ${LINUX_INIT_SERVICE_FILE} \
        || errorExit "Failed to copy init script template, command [ cp -f ${LINUX_INIT_SERVICE_FILE} ${LINUX_INIT_SERVICE_FILE}.init.backup ]"

    chmod a+x ${LINUX_INIT_SERVICE_FILE} \
        || errorExit "Failed to make init file executable, command [ chmod a+x ${LINUX_INIT_SERVICE_FILE} ]"

    sed -i -e "
            s,__JF_PRODUCT_HOME__,${JF_PRODUCT_HOME},g;
            s,__SERVICE__,$LINUX_SERVICE_NAME,g" ${LINUX_INIT_SERVICE_FILE} \
        || errorExit "Failed to home and service name in ${LINUX_INIT_SERVICE_FILE} script"

    # Try update-rc.d for debian/ubuntu else use chkconfig
    if [ -x /usr/sbin/update-rc.d ]; then
        update-rc.d ${LINUX_SERVICE_NAME} defaults &>/dev/null && \
        chkconfigOK=true
    elif [ -x /usr/sbin/chkconfig ] || [ -x /sbin/chkconfig ]; then
        chkconfig --add ${LINUX_SERVICE_NAME} &>/dev/null && \
        chkconfig ${LINUX_SERVICE_NAME} on &>/dev/null && \
        chkconfig --list ${LINUX_SERVICE_NAME} &>/dev/null && \
        chkconfigOK=true
    else
        ln -s ${LINUX_INIT_SERVICE_FILE} /etc/rc3.d/S99${LINUX_SERVICE_NAME} && \
        chkconfigOK=true
    fi
    [ ${chkconfigOK} ] || errorExit "Service ${LINUX_SERVICE_NAME} installation failed"
}

linux_writeUserInfo(){
    rpmDeb_setSystemValueNoOverride ${LINUX_USER_KEY} "${PRODUCT_USER}" "${JF_SYSTEM_YAML}" || true
    rpmDeb_setSystemValueNoOverride ${LINUX_GROUP_KEY} "${PRODUCT_GROUP}" "${JF_SYSTEM_YAML}" || true
}

linux_setownership(){
    io_setOwnership "${JF_PRODUCT_HOME}/app" "${PRODUCT_USER}" "${PRODUCT_GROUP}"
    io_setOwnership "${JF_PRODUCT_HOME}/var/etc" "${PRODUCT_USER}" "${PRODUCT_GROUP}"
}

linux_common_init(){
    : ${LINUX_USER_KEY:="shared.user"}
    : ${LINUX_GROUP_KEY:="shared.group"}
    : ${JF_SYSTEM_YAML:="${JF_PRODUCT_HOME}/var/etc/system.yaml"}
}

linux_installService_main(){
    linux_installService_processOptions $*
    
    SYSTEM_DIAGNOSTICS_LOG_FILE="${JF_PRODUCT_HOME}/var/log/systemDiagnostics.log"
    
    linux_common_init

    rpmDeb_checkPrereqs

    setUserGroup "${LINUX_USER_KEY}" "${LINUX_GROUP_KEY}" "${PRODUCT_USER}" "${PRODUCT_GROUP}"

    createUserAndGroup

    linux_setProductHome "${LINUX_SERVICE_DEFAULT_ENV}"

    linux_createRunDir

    linux_installService

    linux_writeUserInfo

    linux_setownership

    displayRpmDebStartInfo "${LINUX_SERVICE_NAME}" "${PRODUCT_NAME}"
}

linux_uninstallService_main(){
    linux_common_init

    validateSudo

    linux_uninstallService

    linux_displayPostUninstallInfo "${LINUX_SERVICE_NAME}" "${PRODUCT_NAME}"    
}

linux_displayPostUninstallInfo(){
    logger "Uninstallation of service ${LINUX_SERVICE_NAME} completed"
}

linux_main() {

    linux_processOptions $*

    linux_init

    rpmDeb_setYQPath

    setUserGroup "${LINUX_USER_KEY}" "${LINUX_GROUP_KEY}" "${PRODUCT_USER}" "${PRODUCT_GROUP}"

    createUserAndGroup

    linux_createLogFile
    
    rpmDeb_setMaxOpenfiles

    rpmDeb_checkPrereqs

    bannerStart "Beginning JFrog ${PRODUCT_NAME} setup"

    rpmDeb_startLog
    
    linux_preUserInputChecks

    rpmDeb_getUserInputs
    
    linux_askPostgresdetails

    rpmDeb_writeInfoToYaml

    linux_writeUserInfo

    linux_setOwnership

    rpmDeb_cleanup

    linux_endInfo
}

linux_main_func(){
    linux_main $*
}