WRAPPER_SCRIPT_TYPE="$WRAPPER_SCRIPT_TYPE_RPMDEB"

rpmDeb_isUseDefaultsEnabled() {
    if [[ "${USE_DEFAULTS}" == true ]]; then
        return 0
    else
        return 1
    fi
}

rpmDeb_isNotUpgrade() {
    if [[ $RPM_DEB_IS_UPGRADE != true ]]; then
        return 0
    else
        return 1
    fi
}

#check if the script can be run
rpmDeb_checkLinuxDistribution() {
    validateSudo
    io_getLinuxDistribution

    # If not set - not supported
    if [ -z "${LINUX_DISTRIBUTION}" ]; then errorExit "Linux distribution type not supported"; fi

    if [[ "${LINUX_DISTRIBUTION}" == "Debian" ]] && [[ "${PRODUCT_NAME}" == "Distribution" ]] && [[ $(cat /etc/*release 2>/dev/null | grep "VERSION_ID=\"9\"" 2>/dev/null) == *"VERSION_ID"* ]];
    then
        errorExit "${LINUX_DISTRIBUTION} 9 is not supported."
    fi

    if [[ "${LINUX_DISTRIBUTION}" != "Debian" && "${LINUX_DISTRIBUTION}" != "AmazonLinux" && "${LINUX_DISTRIBUTION}" != "Ubuntu" && "${LINUX_DISTRIBUTION}" != "CentOS" && "${LINUX_DISTRIBUTION}" != "RedHat" && "${LINUX_DISTRIBUTION}" != "SLES" ]];
    then
        errorExit "${LINUX_DISTRIBUTION} is not supported. Debian, Ubuntu, CentOS, RedHat, Amazon Linux 3 and its variants are supported"
    fi

    # Remove LC_CTYPE from env, fix for mac users (via ssh), which execute the command with sudo
    if [ ! -z ${LC_CTYPE} ]; then
        LC_CTYPE_BACKUP=${LC_CTYPE}
        unset LC_CTYPE
    fi
}

# Returned code:
#0) $1=$2
#1) $1>$2
#2) $1<$2
rpmDeb_verComp () {
    if [[ $1 == $2 ]]
    then
        return 0
    fi
    local IFS=.
    local i
    local ver1=($1)
    local ver2=($2)
    # fill empty fields in ver1 with zeros
    for ((i=${#ver1[@]}; i<${#ver2[@]}; i++))
    do
        ver1[i]=0
    done
    for ((i=0; i<${#ver1[@]}; i++))
    do
        if [[ -z ${ver2[i]} ]]
        then
            # fill empty fields in ver2 with zeros
            ver2[i]=0
        fi
        if ((10#${ver1[i]} > 10#${ver2[i]}))
        then
            return 1
        fi
        if ((10#${ver1[i]} < 10#${ver2[i]}))
        then
            return 2
        fi
    done
    return 0
}

# Check if a tool is installed by accepting packname as a parameter
# Will return 1 if it is not installed
# usage : rpmDeb_isInstalled "curl"
rpmDeb_isInstalled(){
    local packName=$1
    local name=''
    local version=''
    
    arrIN=(${packName//=/ })
    packName=${arrIN[0]}

    if [ -z ${packName} ] || [ "${packName}" == "" ]
    then
        errorExit "isInstalled: missing packName value"
    fi

    case ${LINUX_DISTRIBUTION} in
        CentOS|RedHat|SLES|AmazonLinux)
            name="$(rpm -qa --nosignature --queryformat '%{NAME}\n' ${packName} 2>/dev/null)"
            version="$(rpm -qa --nosignature --queryformat '%{VERSION}\n' ${packName} 2>/dev/null)"
        ;;
        Debian|Ubuntu)
            name=$(dpkg-query -W -f='${binary:Package}' ${packName} 2>/dev/null)
            version=$(dpkg-query -W -f='${Version}' ${packName} 2>/dev/null)
        ;;
        *)
            errorExit "isInstalled: Distribution ${LINUX_DISTRIBUTION} is not supported"
            return 1
        ;;
    esac

    # if name and version is not available consider it as not installed and return 1
    # let the caller decide what to do based on this response
    if [ -z "${name}" ] || [ "${name}" == "" ] || [ -z "${version}" ] || [ "${version}" == "" ]
    then
        return 1
    else
        return 0
    fi
}

rpmDeb_isPackInstalled() {
    local packName=$1
    local packFile=$2
    local strictCheck=$3
    if [[ ! -f "$packFile" ]]
    then
        errorExit "File ${packFile} not found, please verify if all the extracted files are available. If not extract and try again"
    fi
    local currentName=''
    local newName=''
    local currentVer=''
    local newVer=''
    local currentRel=''
    local newRel=''
    local productPackName='${RPM_DEB_PRODUCT_PACK_NAME}'

    if [[ -z "${packName}" ]] || [[ "${packName}" == "" ]] || [[ -z "${packFile}" ]] || [[ "${packFile}" == "" ]]
    then
        errorExit "Method rpmDeb_isPackInstalled: missing packName\packFile value"
    fi

    case ${LINUX_DISTRIBUTION} in
        CentOS|RedHat|SLES|AmazonLinux)
            currentName="$(rpm -qa --nosignature --queryformat '%{NAME}\n' ${packName} 2>/dev/null)"
            currentVer="$(rpm -qa --nosignature --queryformat '%{VERSION}\n' ${packName} 2>/dev/null)"
            currentRel="$(rpm -qa --nosignature --queryformat '%{RELEASE}\n' ${packName} 2>/dev/null)"

            newName="$(rpm -qp --nosignature --queryformat '%{NAME}\n' ${packFile} 2>/dev/null)"
            newVer="$(rpm -qp --nosignature --queryformat '%{VERSION}\n' ${packFile} 2>/dev/null)"
            newRel="$(rpm -qp --nosignature --queryformat '%{RELEASE}\n' ${packFile} 2>/dev/null)"
        ;;
        Debian|Ubuntu)
            if [[ "$strictCheck" == *"yes"* ]]; then
                # Check if this is an upgrade
                local status
                local currentState

                # get status of packname
                status=$(dpkg -l "${packName}" 2>/dev/null | grep "${packName}" | awk '{print $1}')
                # get second character of the response
                [[ ! -z $status ]] && currentState=${status:1:1} || currentState="Not Installed"
                
                # if currentState is 'i', consider it as installed
                if [[ "$currentState" == "i" ]]; then
                    currentName=$(dpkg-query -W -f='${binary:Package}' ${packName} 2>/dev/null)
                    currentVer=$(dpkg-query -W -f='${Version}' ${packName} 2>/dev/null)

                    newName="$(dpkg --info ${packFile} | grep Package | awk -F ': ' '{print $2}')"
                    newVer="$(dpkg --info ${packFile} | grep Version | awk -F ': ' '{print $2}')"
                else
                    return 1
                fi            
            else
                currentName=$(dpkg-query -W -f='${binary:Package}' ${packName} 2>/dev/null)
                currentVer=$(dpkg-query -W -f='${Version}' ${packName} 2>/dev/null)

                newName="$(dpkg --info ${packFile} | grep Package | awk -F ': ' '{print $2}')"
                newVer="$(dpkg --info ${packFile} | grep Version | awk -F ': ' '{print $2}')"
            fi
        ;;
        *)
            errorExit "Method rpmDeb_isPackInstalled: Distribution ${LINUX_DISTRIBUTION} is not supported"
            return 1
        ;;
    esac

    # replace ~ with a dot - version comparison fails for ~
    currentVer=$(echo "${currentVer}" | sed 's/~/./g' )
    newVer=$(echo "${newVer}" | sed 's/~/./g' )

    # replace : with a dot - version comparison fails for :
    currentVer=$(echo "${currentVer}" | sed 's/:/./g' )
    newVer=$(echo "${newVer}" | sed 's/:/./g' )

    # currentVer: to lower, use dots
    currentVer=$(echo "${currentVer,,}" | tr - . )
    currentVer=$(echo "${currentVer,,}" | tr _ . )

    # currentRel: to lower, use dots
    currentRel=$(echo "${currentRel,,}" | tr - . )
    currentRel=$(echo "${currentRel,,}" | tr _ . )

    # newRel: to lower, use dots
    newRel=$(echo "${newRel,,}" | tr - . )
    newRel=$(echo "${newRel,,}" | tr _ . )

    #support for dev upgrades
    if [ "${packName}" == "${productPackName}" ] && [[ "${currentVer}" == *[b]* ]]; then
        currentVer=$(echo "${currentVer,,}" | sed 's/b//g' )
    fi

    # newVer: to lower, use dots
    newVer=$(echo "${newVer,,}" | tr - . )
    newVer=$(echo "${newVer,,}" | tr _ . )

    #support for dev upgrades
    if [ "${packName}" == "${productPackName}" ] && [[ "${newVer}" == *[b]* ]]; then
        newVer=$(echo "${newVer,,}" | sed 's/b//g' )
    fi

    #support for any version which contains string 
    # example : 2.2.0-m001
    #           3.0.0-rc001
    if [ "${packName}" == "${productPackName}" ] && [[ "${currentVer}" =~ ~*-*[a-zA-Z]+ ]]; then
        currentVer=$(echo "${currentVer,,}" | sed 's/-*~*[a-z][a-zA-Z]*/./g' )
        #handle if multiple dots are generated
        currentVer=$(echo "${currentVer}" | sed 's/\.\.\.*/./g' )
    fi

    #support for any version which contains string 
    # example : 2.2.0-m001
    #           3.0.0-rc001
    if [ "${packName}" == "${productPackName}" ] && [[ "${newVer}" =~ ~*-*[a-zA-Z]+ ]]; then
        newVer=$(echo "${newVer,,}" | sed 's/-*~*[a-z][a-zA-Z]*/./g' )
        #handle if multiple dots are generated
        newVer=$(echo "${newVer}" | sed 's/\.\.\.*/./g' )
    fi

    if [ "${currentName}" == "${newName}" ]; then
        rpmDeb_verComp "${currentVer}" "${newVer}" 2>/dev/null
        verCompResult=$?
        if [ ${verCompResult} -eq 0 ] || [ ${verCompResult} -eq 1 ]; then
            # compare release numbers
            rpmDeb_verComp "${currentRel}" "${newRel}" 2>/dev/null
            relCompResult=$?
            
            if [ ${relCompResult} -eq 0 ] || [ ${relCompResult} -eq 1 ]; then
                logger "Packname ${packName} [$currentVer] version is already installed" >&6 2>&1
            elif [ ${relCompResult} -eq 2 ]; then
                logger "Packname ${packName} [$currentVer] ($currentRel) version is installed, going to upgrade to [$newVer] ($newRel) version" >&6 2>&1
                return 2
            fi

        elif [ ${verCompResult} -eq 2 ]; then
            [ -z "$currentVer" ] || \
                logger "Packname ${packName} [$currentVer] version is installed, going to upgrade to [$newVer] version"
            return 2
        fi
    else
        # This version is not installed
        return 1
    fi

    return 0
}

# Extract package name by using package file as input
# usage : rpmDeb_getPackName "libvpx1_1.3.0-3_amd64.deb"
rpmDeb_getPackName() {
    local file="$1"
    if [ ! -f $file ]
    then
        errorExit "${file} not found, please verify if all the extracted files are available. If not extract and try again"
    fi
    if [ ! -z $file ]; then
        if [[ $file == *.deb ]]; then
            echo $(dpkg --info ${file} | grep Package | awk -F ': ' '{print $2}')
        elif [[ $file == *.rpm ]]; then
            echo $(rpm -qp --nosignature --queryformat '%{NAME}\n' ${file} 2>/dev/null)
        fi
    else
        echo ""
    fi
}

rpmDeb_invokeSystemUnInstaller() {
    packName=$1
    case ${LINUX_DISTRIBUTION} in
    CentOS|RedHat|SLES|AmazonLinux)
        yum erase -y $packName 2>/dev/null
    ;;
    Debian|Ubuntu)
        DEBIAN_FRONTEND=noninteractive apt purge -y $packName 2>/dev/null
    ;;
    *)
        errorExit "Distribution ${LINUX_DISTRIBUTION} is not supported"
    ;;
    esac
}

# Generic method to install deb/rpm package using package name
# parameters :
#   packFile        file to be installed
#   strictCheck      if yes is passed, a thorough check of install will be done
#   mode             install/upgrade , by default install will be executed
#   upgradeMode      set this to skip if you do not want to upgrade if installed
#   exitOnFailure    set to yes by default, set it no or anything else if you to continue on failure
#   extraOptions     pass any extra flags to be passed during install
# usage : rpmDeb_installPackage "somepackage.deb" "lenient_check" "install_mode" "force_upgrade" "yes" "--force-confnew"
rpmDeb_installPackage() {
    local packFile=$1
    local strictCheck=$2
    local mode=$3
    local upgradeMode=$4
    local exitOnFailure=${5:-yes}
    local extraOptions=${6:-""}
    local rpmOptions

    # run upgrade of rpm if 'upgrade' is passed as the third argument
    [[ ! -z "$mode" ]] && [[ "$mode" == *"upgrade"* ]] && rpmOptions="Uvh" || rpmOptions="ivh"

    if [ ! -f "${packFile}" ]
    then
        ls "${packFile}"
        errorExit "File ${packFile} not found, please verify if all the extracted files are available. If not extract and try again"
    fi

    logDebug "Installing pack file: $packFile"
    local packName=$(rpmDeb_getPackName ${packFile})
    logDebug "Installing packName: $packName"

    [ ! -z "${packName}" ] || errorExit "Installing ${packFile} failed, could not determine packname using package"

    rpmDeb_isPackInstalled "${packName}" "${packFile}" "${strictCheck}"

    if [ $? -eq 0 ]; then
        logger "Skipping ${packName} installation" >&6 2>&1
    elif [[ $? -eq 2 ]] && [[ ! -z "$upgradeMode" ]] && [[ "$upgradeMode" == *"skip"* ]]; then 
        logger "Skipping ${packName} installation" >&6 2>&1
    elif [[ $packFile == *.deb ]]; then
        logDebug "Installing as deb"
        dpkg -i ${extraOptions} ${packFile} >&6
        if [[ $? -ne 0 ]]; then
            [[ $exitOnFailure == "yes" ]] && errorExit "Installing ${packName} package failed" || echo "Installing ${packName} package failed, port might already be allocated" >&6
        fi
    elif [[ $packFile == *.rpm ]]; then
        logDebug "Installing as rpm"
        rpm -${rpmOptions} ${extraOptions} --nosignature --replacepkgs ${packFile} >&6
        if [[ $? -ne 0 ]]; then
            [[ $exitOnFailure == "yes" ]] && errorExit "Installing ${packName} package failed" || echo "Installing ${packName} package failed, port might already be allocated" >&6
        fi
    fi
}

# usage : rpmDeb_removePackage "package name" 
rpmDeb_removePackage() {
    local packageName=$1
    logger "Removing ${packageName} ..."
    case ${LINUX_DISTRIBUTION} in
        CentOS|RedHat|SLES|AmazonLinux)
            rpm -e ${packageName}
        ;;
        Debian|Ubuntu)
            dpkg -r ${packageName}
        ;;
        *)
            errorExit "Distribution ${LINUX_DISTRIBUTION} is not supported"
        ;;
    esac
}

# Attempts to start services
rpmDeb_startService() {
    local packName=$1

    logger "Starting ${packName}"
    local status_command
    local start_command
    #but systemctl may not exist
    if [ -f "/etc/init.d/${packName}" ] || [ -f "/lib/systemd/system/${packName}.service" ] ; then
        status_command="service ${packName} status"
        start_command="service ${packName} start"
    elif [ -f "/usr/lib/systemd/system/${packName}.service" ] || [ -f "/etc/systemd/system/${packName}.service" ]; then
        status_command="systemctl status ${packName}"
        start_command="systemctl start ${packName}"
    fi
    ($status_command || $start_command) >&6 2>&1

    logDebug "Command to check status: $status_command"

    local retry_count=0
    local attempstat=1
    while [ ${retry_count} -lt 10 ] && [ ${attempstat} -gt 0 ]; do
        sleep 2
        $status_command >&6 2>&1
        attempstat=$?
        let retry_count=$retry_count+1
    done
    if [ ${attempstat} -gt 0 ]; then
        logError "failed to start [$packName]. This may be temporary. Try manually starting the service using [$start_command] and "
        errorExit "verifying its status using [$status_command]. Once it starts successfully, rerun the installer"
    fi
}

rpmDeb_checkPrerequisitesSoftware() {
    # Validate that any pre-requisites specified by the 3rd party libraries are present
    local USER_END_INSTALLS=$1
    local unavailableTools=""
    local toolList=($USER_END_INSTALLS)

    for packName in "${toolList[@]}"
    do
       rpmDeb_isInstalled "${packName}"
       if [ $? -eq 1 ] ; then
          unavailableTools="${unavailableTools} ${packName}"
       fi
    done

    local command_to_run
    case ${LINUX_DISTRIBUTION} in
        CentOS|RedHat|SLES|AmazonLinux)
            command_to_run="yum install -y ${unavailableTools}"
        ;;
        Debian|Ubuntu)
            command_to_run="apt-get install -y ${unavailableTools}"
        ;;
    esac

    if [ ! -z "${unavailableTools}" ]; then
        logError "Some dependencies the installer needs are missing"
        logError "Please install them using [${command_to_run}] and then try again"
        errorExit "NOTE: You may need to tweak the list of sources in some cases"
    fi
}

configureLogOutput() {
    if [[ ! -z $INSTALLATION_LOG_FILE ]]; then
        exec 6>>$INSTALLATION_LOG_FILE
    else
        exec 6>&1
    fi

}

rpmDeb_setYQPath() {
    logDebug "Method rpmDeb_setYQPath"
    if [ ! -z "${RPM_DEB_INSTALLER_HOME}" ] && [ -d "${RPM_DEB_INSTALLER_HOME}" ]; then
        _findYQ "${RPM_DEB_INSTALLER_HOME}"
    fi
    
    if [ -z "${YQ_PATH}" ] && [ ! -z "${JF_PRODUCT_HOME}" ] && [ -d "${JF_PRODUCT_HOME}" ]; then
        _findYQ "${JF_PRODUCT_HOME}"
    fi
}

rpmDeb_createLogFile() {
    redirectLogsToFile #creates variable DESTINATION_LOG_FILE
    INSTALLATION_LOG_FILE="${RPM_DEB_INSTALLER_HOME}/install.log"
    configureLogOutput
}

rpmDeb_getPackVersion(){
    local packName=$1

    [ ! -z "${packName}" ] || return 0

    local currentVer=
    case ${LINUX_DISTRIBUTION} in
        CentOS|RedHat|SLES|AmazonLinux)
            currentVer="$(rpm -qa --nosignature --queryformat '%{VERSION}\n' ${packName} 2>/dev/null)"
        ;;
        Debian|Ubuntu)
            currentVer=$(dpkg-query -W -f='${Version}' ${packName} 2>/dev/null)
        ;;
        *)
            errorExit "Distribution $LINUX_DISTRIBUTION is not supported"
        ;;
    esac
    echo "${currentVer}"
}

rpmDeb_setVarSymlink(){
    local symlink="${JF_PRODUCT_HOME}/var"

    # Readlink will give the same path as response if it does not exist
    local varDir=$(readlink -f "${symlink}" 2>/dev/null)

    if [[ "${varDir}" == *"${symlink}"* ]]; then
        logger "Symlink for data directory not found" >&6 2>&1
    else
        RPM_DEB_DATA_DIR="${varDir}"
        return 0
    fi
}

rpmDeb_setProductDataEnv(){
    local dataDir="${RPM_DEB_DATA_DIR}"

    # If we are unable to discover current data directory by ourself, get it from installer yaml 
    if [ -z "${dataDir}" ]; then
        getSystemValue "${KEY_DATA_DIR_LOCATION}" "NOT_SET" "false" "${INSTALLER_YAML}"
        dataDir="${YAML_VALUE}"
    fi

    if [ -z "${dataDir}" ] || [ "${dataDir}" == "NOT_SET" ]; then
        return 0
    else
        addOrReplaceProperty "export JF_PRODUCT_VAR" "${dataDir}" "${RPM_DEB_DEFAULT_ENV}" "="
    fi
}

rpmDeb_installProduct() {
    local packFile=""
    bannerStart "${PRODUCT_NAME} Installation"
    case ${LINUX_DISTRIBUTION} in
        CentOS|RedHat|SLES|AmazonLinux)
            packFile=$(ls ${RPM_DEB_PRODUCT_FILE}*.rpm)
        ;;
        Debian|Ubuntu)
            packFile=$(ls ${RPM_DEB_PRODUCT_FILE}*.deb)
        ;;
        *)
            errorExit "Distribution ${LINUX_DISTRIBUTION} is not supported"
        ;;
    esac

    logger "Installing/Verifying ${PRODUCT_NAME} (${packFile}) (this may take several minutes)..."

    # pass 'yes' do a thorough check before install 
    rpmDeb_installPackage "${packFile}" "yes" "upgrade"

    rpmDeb_setProductDataEnv
}

rpmDeb_isUpgrade() {
    local packName=$1

    [ ! -z "${packName}" ] || return

    case ${LINUX_DISTRIBUTION} in
        CentOS|RedHat|SLES|AmazonLinux)
            local currentVersion=$(rpm -qa --nosignature --queryformat '%{VERSION}\n' ${packName} 2>/dev/null)
            if [[ ! -z "${currentVersion}" && "${currentVersion}" != "" ]]; then
                RPM_DEB_IS_UPGRADE=true
                IS_UPGRADE="${FLAG_Y}"
                RPM_DEB_PRODUCT_INSTALLED_VER="${currentVersion}"
            fi
        ;;
        Debian|Ubuntu)
            # Check if this is an upgrade
            local status
            local currentState

            # get status of packname
            status=$(dpkg -l "${packName}" 2>/dev/null | grep "${packName}" | awk '{print $1}')
            # get second character of the response
            [[ ! -z $status ]] && currentState=${status:1:1} || currentState="Not Installed"
            
            # if currentState is 'i', consider it as installed
            if [[ "$currentState" == "i" ]]; then
                RPM_DEB_IS_UPGRADE=true
                IS_UPGRADE="${FLAG_Y}"
                RPM_DEB_PRODUCT_INSTALLED_VER=$(dpkg-query -W -f='${Version}' ${packName} 2>/dev/null)
            fi
        ;;
        *)
            errorExit "Distribution ${LINUX_DISTRIBUTION} is not supported"
        ;;
    esac

    if [ "${RPM_DEB_IS_UPGRADE}" == "true" ] && [ ! -z "${RPM_DEB_MIN_VERSION}" ] && [ ! -z "${RPM_DEB_MIN_VERSION_ERR_MSG}" ]; then
        rpmDebIsUpgradeAllowed "${packName}" "${RPM_DEB_MIN_VERSION}" "${RPM_DEB_MIN_VERSION_ERR_MSG}"
    fi 
}

rpmDeb_usage () {
    echo "Usage: ${0}"
    cat << END_USAGE

-v | --verbose                                    : [optional] run installer in verbose mode
-h | --help                                       : [optional] display usage

END_USAGE
    sleep .5
}

rpmDeb_processOptions() {
    VERBOSE_MODE=false
                
    while [[ $# > 0 ]]; do
        case "$1" in
            -h | --help)
                rpmDeb_usage
                exit 0
            ;;
            *)
                rpmDeb_usage
                exit 1
            ;;
        esac
    done
}

rpmDeb_startLog(){
    logger "This script will install ${PRODUCT_NAME} and its dependencies."
    logger "After installation, logs can be found at ${INSTALLATION_LOG_FILE}"
}

rpmDeb_checkUserEndInstalls(){
    local userEndInstalls
    case ${LINUX_DISTRIBUTION} in
        CentOS|RedHat|SLES|AmazonLinux)
            userEndInstalls="$RPM_DEB_THIRDPARTY_DEP_CENTOS"
        ;;
        Debian)
            userEndInstalls="$RPM_DEB_THIRDPARTY_DEP_DEBIAN"
        ;;
        Ubuntu)
            userEndInstalls="$RPM_DEB_THIRDPARTY_DEP_UBUNTU"
        ;;
    esac
	rpmDeb_checkPrerequisitesSoftware "$userEndInstalls"    
}

rpmDeb_thirdPartyInstall (){
    for installCommand in ${RPM_DEB_THIRDPARTY_LIBRARIES}
    do
        logger "Invoking ${installCommand}" >&6 2>&1
        eval "${installCommand}"
    done
}

rpmDeb_endInfo(){
    echo ""
    logger "Installation complete. Installation log can be found at [${INSTALLATION_LOG_FILE}]"

    displayRpmDebStartInfo "${RPM_DEB_PRODUCT_SERVICE_NAME}" "${PRODUCT_NAME}"

    getJFrogURLIfAvailable >/dev/null || true

    echo ""
    if [ ! -z "${JF_SHARED_JFROGURL}" ]; then
        logger "Once the application has started, it can be accessed at [${JF_SHARED_JFROGURL}]"
    fi
    echo ""
}

rpmDeb_copyYaml(){
    local mode="${1:-"pre"}"
    local installerYaml="${FILE_NAME_INSTALLER_YAML}"
    local systemYaml="system.yaml"

    local dataDir="${JF_PRODUCT_HOME}/var/etc/"

    if [ "${mode}" == "pre" ]; then
        [ -f "${dataDir}/${installerYaml}" ] && copyFile "${dataDir}/${installerYaml}" "${INSTALLER_YAML}" || true
    elif [ "${mode}" == "post" ]; then
        [ -f "${INSTALLER_YAML}" ] && copyFile "${INSTALLER_YAML}" "${dataDir}/${installerYaml}" || true
    else
        warn "Unsupported mode passed to rpmDeb_copyYaml"
    fi
}

rpmDeb_intializeLogFileName(){
    local parentPath=${1:-"${SCRIPT_HOME}"}
    local scriptHome="$(basename $0 | awk -F '.' '{print $1}')"
    local currentDate=$(date +"%Y%m%d%H%M")
    local logFileName="${scriptHome}.${currentDate}.log"
    INSTALLATION_LOG_FILE="${parentPath}/${logFileName}"
}

rpmDeb_init(){
    RPM_DEB_IS_UPGRADE="false"
    RPM_DEB_KEY_SYSTEM_TO_INSTALLER="systemToInstaller"
    RPM_DEB_KEY_INSTALLER_TO_SYSTEM="installerToSystem"
    JF_SYSTEM_YAML=${JF_PRODUCT_HOME}/var/etc/"${FILE_NAME_SYSTEM_YAML}"
    INSTALLER_STATE_YAML_FILE=${JF_PRODUCT_HOME}/var/etc/"${FILE_NAME_INSTALLER_STATE_YAML}"
}

rpmDeb_getUserInputs(){
    wrapper_getUserInputs
}

rpmDeb_hook_getUserInputs(){
    logDebug "rpmDeb_hook_getUserInputs (unimplemented)"
}

rpmDeb_ha_hook_getUserInputs(){
    logDebug "rpmDeb_ha_hook_getUserInputs (unimplemented)"
}

rpmDeb_hook_start_getUserInputs(){
    logDebug "rpmDeb_hook_start_getUserInputs (unimplemented)"
}

# Each product should try to discover data directory and set RPM_DEB_DATA_DIR with its value
# This will later be used to prompt user for this info or not
rpmDeb_hook_discoverDataDir(){
    logDebug "rpmDeb_hook_discoverDataDir (unimplemented)"
}

wrapper_hook_start_getUserInputs(){
    rpmDeb_hook_start_getUserInputs
    if [ -z "${RPM_DEB_DATA_DIR}" ]; then
        wrapper_askUser "DATA_DIR_LOCATION"
    fi
}

wrapper_hook_getUserInputs(){
    rpmDeb_hook_getUserInputs
}

wrapper_ha_hook_getUserInputs(){
    rpmDeb_ha_hook_getUserInputs
}

rpmDeb_hook_setInstallerDefaults(){
    logDebug "rpmDeb_hook_setInstallerDefaults (unimplemented)"
}

wrapper_hook_setInstallerDefaults(){
    local dataDir=${RPM_DEB_DATA_DIR:-"${JF_PRODUCT_VAR_DEFAULT}"}
    wrapper_setSystemValue "${KEY_DATA_DIR_LOCATION}" "${dataDir}" "${FLAG_N}"
    rpmDeb_hook_setInstallerDefaults
}

rpmDeb_PostgresInfoToYaml(){
    local forceSync="${1}"
    local type="${RPM_DEB_KEY_INSTALLER_TO_SYSTEM}"

    if [[ "${PRODUCT_NAME}" == "${CATALOG_LABEL}" ]]; then
        rpmDeb_setSystemValueNoOverride "${SYS_KEY_SHARED_DATABASE_TYPE}"   "postgresql" "${JF_SYSTEM_YAML}"
        rpmDeb_setSystemValueNoOverride "${SYS_KEY_SHARED_DATABASE_DRIVER}" "pgx" "${JF_SYSTEM_YAML}"
    fi
    rpmDeb_syncYaml "${KEY_DATABASE_URL}"      "${SYS_KEY_SHARED_DATABASE_URL}"      "${type}" || warn "Could not set ${SYS_KEY_SHARED_DATABASE_URL}"
    rpmDeb_syncYaml "${KEY_DATABASE_USERNAME}" "${SYS_KEY_SHARED_DATABASE_USERNAME}" "${type}" || warn "Could not set ${SYS_KEY_SHARED_DATABASE_USERNAME}"
    rpmDeb_syncYaml "${KEY_DATABASE_PASSWORD}" "${SYS_KEY_SHARED_DATABASE_PASSWORD}" "${type}" || warn "Could not set ${SYS_KEY_SHARED_DATABASE_PASSWORD}"
}

rpmDeb_syncYaml() {
    local keyToFetch=${1}
    local keyToUpdate=${2}
    local valueToUpdate=""
    local type=${3:-"${RPM_DEB_KEY_INSTALLER_TO_SYSTEM}"}
    
    local src="${4:-${INSTALLER_YAML}}"
    local target="${5:-${JF_SYSTEM_YAML}}"

    # Return if any of the inputs are empty
    [[ -z "$keyToFetch"    || "$keyToFetch"  == "" ]] && return 1
    [[ -z "$keyToUpdate"   || "$keyToUpdate"  == ""   ]] && return 1

    if [ "${type}" == "${RPM_DEB_KEY_SYSTEM_TO_INSTALLER}" ]; then
        src="${JF_SYSTEM_YAML}"
        target="${INSTALLER_YAML}"
    fi

    getSystemValue "${keyToFetch}" "NOT_SET" "false" "${src}"
    valueToUpdate="${YAML_VALUE}"

    if [ "${valueToUpdate}" == "NOT_SET" ]; then
        return 0
    fi

    if [[ "${target}" == "${JF_SYSTEM_YAML}" ]] || [[ "${target}" == "${INSTALLER_STATE_YAML_FILE}" ]]; then
        rpmDeb_setSystemValueNoOverride "${keyToUpdate}" "${valueToUpdate}" "${target}"
    else
        setSystemValue "${keyToUpdate}" "${valueToUpdate}" "${target}"
    fi
}

rpmDeb_setSystemValueNoOverride(){
    local key=$1
    local value=$2
    local yamlFile=$3
    local logMsg=${4:-false}

    # Return if any of the inputs are empty
    [[ -z "$key"      || "$key"      == "" ]] && return
    [[ -z "$value"    || "$value"    == "" ]] && return
    [[ -z "$yamlFile" || "$yamlFile" == "" ]] && return

    getSystemValue "${key}" "NOT_SET" "${logMsg}" "${yamlFile}"

    if [ ${YAML_VALUE} == "NOT_SET" ]; then
        setSystemValue "${key}" "${value}" "${yamlFile}"
    else
        [ "${logMsg}" == "true" ] && echo "Skipping update of yaml key : ${key}" || true
    fi
}

rpmDeb_mandatoryInfoToYaml(){
    local type="${RPM_DEB_KEY_INSTALLER_TO_SYSTEM}"
    rpmDeb_syncYaml "${KEY_JFROGURL}"  "${SYS_KEY_SHARED_JFROGURL}"           "${type}" || warn "Could not set ${SYS_KEY_SHARED_JFROGURL}"    
    rpmDeb_syncYaml "${KEY_JOIN_KEY}"  "${SYS_KEY_SHARED_SECURITY_JOINKEY}"   "${type}" || warn "Could not set ${SYS_KEY_SHARED_SECURITY_JOINKEY}"
    rpmDeb_syncYaml "${KEY_CLUSTER_MASTER_KEY}"  "${SYS_KEY_SHARED_SECURITY_MASTERKEY}"   "${type}" || warn "Could not set ${SYS_KEY_SHARED_SECURITY_MASTERKEY}"
    rpmDeb_syncYaml "${KEY_HOST_IP}"   "${SYS_KEY_SHARED_NODE_IP}"            "${type}" || warn "Could not set ${SYS_KEY_SHARED_NODE_IP}"
    setSystemValue "router.entrypoints.internalPort" "7046" "$JF_SYSTEM_YAML"
    setSystemValue "router.entrypoints.externalPort" "7082" "$JF_SYSTEM_YAML"
    setSystemValue "router.entrypoints.traefikApiPort" "7049" "$JF_SYSTEM_YAML"
    setSystemValue "router.entrypoints.grpcPort" "7047" "$JF_SYSTEM_YAML"
    setSystemValue "catalog.mode" "singleTenant" "$SYSTEM_YAML_FILE"
    setSystemValue "catalog.central.url" "https://jfscatalogcentral.jfrog.io" "$SYSTEM_YAML_FILE"
    setSystemValue "catalog.logging.application.level" "info" "$SYSTEM_YAML_FILE"
    setSystemValue "shared.node.id" "catalog-$(io_getPublicHostIP)" "$JF_SYSTEM_YAML"
    createMasterkeyFile
}

rpmDeb_InstallerStateInfoToYaml(){
    local type="INSTALLER_TO_INSTALLER_STATE"

    local stateIndicatorValue=""
    setInstallerStateKeys
    for stateIndicatorKey in ${INSTALLER_STATE_KEYS}; do
        eval stateIndicatorValue=\$${stateIndicatorKey}
        [ ! -z "${stateIndicatorValue}" ] || continue
        rpmDeb_syncYaml "${stateIndicatorValue}" "${stateIndicatorValue}" "${type}" "${INSTALLER_YAML}" "${INSTALLER_STATE_YAML_FILE}" \
            || warn "Could not set ${stateIndicatorValue}"
    done
}

createMasterkeyFile() {
    getSystemValue "${KEY_CLUSTER_MASTER_KEY}" "NOT_SET" "false" "${INSTALLER_YAML}"
    local masterKeyValue="${YAML_VALUE}"
    [[ -z "${masterKeyValue}" || "${masterKeyValue}" == "" || "${masterKeyValue}" == "NOT_SET" ]] && return
    masterKeyFile="$JF_PRODUCT_HOME/var/etc/security/master.key"
    createDir "${JF_PRODUCT_HOME}/var/etc/security" || warn "Could not create ${JF_PRODUCT_HOME}/var/etc/security directory"
    echo -n "${masterKeyValue}" > "${masterKeyFile}" || warn "Could not create ${masterKeyFile}"
}


rpmDeb_thirdPartyInfoToYaml(){
    rpmDeb_PostgresInfoToYaml
}

rpmDeb_checkPrereqs(){
    validationCheckUMask
    rpmDeb_checkLinuxDistribution
    rpmDeb_runHardwareRequirementChecks
    rpmDeb_preReqInfo
    #validationCheckPrerequisites
    #rpmDeb_validateDiskSpace
    rpmDeb_checkUserEndInstalls
}

rpmDeb_setMaxOpenfiles() {
    if [[ "${PRODUCT_NAME}" == "${CATALOG_LABEL}" ]]; then
        local noFileVal=$(ulimit -n)
        local minNoFileMax=100000
        if [[ "$noFileVal" != "unlimited" ]] && [[ $noFileVal -lt $minNoFileMax ]]; then
            if [[ -f "/etc/security/limits.conf" ]]; then
                addLine "root hard nofile 100000" "/etc/security/limits.conf"
                addLine "root soft nofile 100000" "/etc/security/limits.conf"
                addLine "catalog hard nofile 100000" "/etc/security/limits.conf"
                addLine "catalog soft nofile 100000" "/etc/security/limits.conf"
            fi
            [[ -f "/etc/pam.d/common-session" ]] && addLine "session required pam_limits.so" "/etc/pam.d/common-session"
        fi
    fi
}

rpmDeb_validateDiskSpace(){
    if [ ! -z "${JF_PRODUCT_HOME}" ]; then
        if [ -L "${JF_PRODUCT_HOME}/var" ]; then
            local targetLink=$(readlink -f "${JF_PRODUCT_HOME}/var")
            validateDiskSpace "${targetLink}"
        else
            [ -d "${JF_PRODUCT_HOME}/var" ] && validateDiskSpace "${JF_PRODUCT_HOME}/var"
        fi
    fi
}
moveSystemYaml(){
    if [ -f "${JF_SYSTEM_YAML}" ]; then
        local jfrogUrl=""
        local databaseUrl=""

        getSystemValue "${SYS_KEY_SHARED_JFROGURL}" "NOT_SET" "false" "${JF_SYSTEM_YAML}"
        jfrogUrl="${YAML_VALUE}"

        getSystemValue "${SYS_KEY_SHARED_DATABASE_URL}" "NOT_SET" "false" "${JF_SYSTEM_YAML}"
        databaseUrl="${YAML_VALUE}"

        # Do not rename system.yaml if,
        #           - jfrog url is already set
        #           - database url is set
        # This will make sure, 
        #       - We are writing to a good content of system.yaml and it already modified atleast once
        #       - There are no empty objects which will result in yq updating it to null
        [ "${jfrogUrl}"    == "NOT_SET" ] || return 0
        [ "${databaseUrl}" == "NOT_SET" ] || return 0

        local backupSystemYaml="${JF_SYSTEM_YAML}.$(date +"%Y%m%d%H%M")"
        logDebug "Found a system yaml ${JF_SYSTEM_YAML}, moving this to ${backupSystemYaml}"
        mv "${JF_SYSTEM_YAML}" "${backupSystemYaml}" >&6 2>&1
    fi
}

rpmDeb_writeInfoToYaml(){
    moveSystemYaml
    rpmDeb_thirdPartyInfoToYaml
    rpmDeb_mandatoryInfoToYaml
    rpmDeb_InstallerStateInfoToYaml
}

rpmDeb_cleanup(){
    [ -f "${INSTALLER_YAML}" ] && rm -f "${INSTALLER_YAML}" >/dev/null 2>&1 || true
}

rpmDeb_main() {
    rpmDeb_init

    rpmDeb_processOptions $*

    rpmDeb_setYQPath

    rpmDeb_createLogFile
    
    rpmDeb_setMaxOpenfiles

    rpmDeb_checkPrereqs
    
    bannerStart "Beginning JFrog ${PRODUCT_NAME} setup"

    rpmDeb_startLog

    rpmDeb_isUpgrade "${RPM_DEB_PRODUCT_PACK_NAME}"

    rpmDeb_hook_discoverDataDir

    rpmDeb_getUserInputs

    rpmDeb_installProduct

    rpmDeb_writeInfoToYaml

    rpmDeb_cleanup

    rpmDeb_endInfo
}
