
displayRpmDebStartInfo () {
    local packName="$1"
    local productName="${2:-product}"

    if [[ -z "${packName}" || -z "${productName}" ]]; then
        return 0
    fi

    local systemctlInfo=$(cat << END_START_USAGE

Start ${productName} with:
> systemctl start ${packName}.service

Check ${productName} status with:
> systemctl status ${packName}.service

END_START_USAGE
) 

    local sysvInfo=$(cat << END_START_USAGE

Start ${productName} with:
> service ${packName} start 

Check ${productName} status with:
> service ${packName} status 
END_START_USAGE
) 

    local isSystemctlAvailable=$(hash systemctl 2>/dev/null; echo $?)

    if [ "${isSystemctlAvailable}" == "0" ]; then
        echo "${systemctlInfo}"
    else
        echo "${sysvInfo}"
    fi

    cat << EXTRA_START_INFO

Installation directory was set to ${JF_PRODUCT_HOME}
You can find more information in the log directory ${JF_PRODUCT_HOME}/var/log

System configuration templates can be found under ${JF_PRODUCT_HOME}/var/etc
Copy any configuration you want to modify from the template to ${JF_PRODUCT_HOME}/var/etc/system.yaml
EXTRA_START_INFO
}

displayArtConnInfo() {
    local doclink="${1}"

    [ ! -z "${doclink}" ] || return 0

    cat << ARTIFACTORY_YAML_INFO

Please note it is mandatory to set ${SYS_KEY_SHARED_JFROGURL} and ${SYS_KEY_SHARED_SECURITY_JOINKEY} in the system.yaml file to complete the configuration.
Both of these are available in this page Admin > Security > Settings
You can read more about these configurations at: ${doclink}
ARTIFACTORY_YAML_INFO
}

rpmDebDetermineType(){
    if dpkg-query --version >/dev/null 2>&1 ; then
        PACKAGE_TYPE="deb"
    elif rpm --version >/dev/null 2>&1; then
        PACKAGE_TYPE="rpm"
    else
        return 1
    fi
}

rpmDebIsUpgradeAllowed(){
    local packName=$1
    local minVersion=$2
    local errMsg="${3:-"Upgrade from version VERSION is not supported"}"
    local name=
    local version=

    if [ -z "${packName}" ] || [ -z "${minVersion}" ] || [ -z "${errMsg}" ]; then
        return 0
    fi

    local arrIN=(${packName//=/ })
    packName=${arrIN[0]}

    rpmDebDetermineType || return 


    case $PACKAGE_TYPE in
        rpm)
            name="$(rpm -qa --nosignature --queryformat '%{NAME}\n' ${packName} 2>/dev/null)"
            version="$(rpm -qa --nosignature --queryformat '%{VERSION}\n' ${packName} 2>/dev/null)"
        ;;
        deb)
            name=$(dpkg-query -W -f='${binary:Package}' ${packName} 2>/dev/null)
            version=$(dpkg-query -W -f='${Version}' ${packName} 2>/dev/null)
        ;;
        *)
            return 0
        ;;
    esac

    # if name and version is not available consider it as not installed and return 1
    # let the caller decide what to do based on this response
    if [ -z "${name}" ] || [ -z "${version}" ]; then
        return 0
    fi

    version=${version//-/.}
    minVersion=${minVersion//-/.}

    version=${version//~/.}
    minVersion=${minVersion//~/.}

    ## This is commented for future use (if the product needs any version compactabilities)
    # if [ "$(io_compareVersions "${version}" "${minVersion}")" == "-1" ]; then
    #     echo
    #     # replace string VERSION in error message to current installed version
    #     errorExit "${errMsg/VERSION/${version}}" >&2
    # fi
}
