
wrapper_setSystemValue() {
    logDebug "Method ${FUNCNAME[0]} Caller:$(caller)"
    io_setYQPath

    local keyToUpdate="$1"
    local valueToUpdate="$2"
    
    local targetFile="${INSTALLER_YAML}"
    local overwrite="${3:-$DONT_PROMPT_USE_DEFAULTS}"

    getYamlValue "$keyToUpdate" "${targetFile}" "false"
    # If a value already exists, check if we need to overwrite
    if [ ! -z "${YAML_VALUE}" ] && [ "${overwrite}" == "$FLAG_N" ]; then
        return 0
    fi
    setSystemValue "$keyToUpdate" "$valueToUpdate" "$targetFile"
}

wrapper_hook_setInstallerDefaults() {
    logSilly "Method ${FUNCNAME[0]} (not implemented)"
}

wrapper_setInstallerDefaults() {
    if [ "$DONT_PROMPT_USE_DEFAULTS" == "$FLAG_Y" ];then
        # Using echo "" > file does not work here. yq complains afterwards
        rm "$INSTALLER_YAML"
        touch "${INSTALLER_YAML}"
    fi

    local overwrite="${FLAG_N}"
    wrapper_setSystemValue "$KEY_ADD_TO_CLUSTER" "$FLAG_N" "${overwrite}"
    wrapper_hook_setInstallerDefaults
    wrapper_setSystemValue "$KEY_HOST_IP" "$(io_getPublicHostIP)" "${overwrite}"
    wrapper_setSystemValue "$KEY_POSTGRES_INSTALL" "$FLAG_N" "${overwrite}"
}

wrapper_createInstallerYaml() {
    logDebug "Method ${FUNCNAME[0]} Caller:$(caller)"

    INSTALLER_YAML=${INSTALLER_YAML:-"${SCRIPT_HOME}/${FILE_NAME_INSTALLER_YAML}"}

    if [ ! -f "$INSTALLER_YAML" ]; then
        logDebug "Creating file: [${INSTALLER_YAML}]"
        touch "${INSTALLER_YAML}"
    fi
}

wrapper_removeInstallerYaml() {
    logDebug "Method ${FUNCNAME[0]} Caller:$(caller)"

    INSTALLER_YAML=${INSTALLER_YAML:-"${SCRIPT_HOME}/${FILE_NAME_INSTALLER_YAML}"}

    if [ -f "$INSTALLER_YAML" ]; then
        logDebug "Removing file: [${INSTALLER_YAML}]"
        rm ${INSTALLER_YAML}
    fi
}

_getRTDataDir() {
    local productHome=
    getYamlValue "$KEY_DATA_DIR_LOCATION" "${INSTALLER_YAML}" "false"
    productHome="$YAML_VALUE"
    if [ -z "${productHome}" ]; then
        return
    elif [ "$WRAPPER_SCRIPT_TYPE" == "$WRAPPER_SCRIPT_TYPE_DOCKER_COMPOSE" ]; then
        productHome="$productHome/${PROJECT_ROOT_FOLDER_RT}/var"
    fi
    echo -n "$productHome"
}

_getDataDir() {
    local productHome=
    getYamlValue "$KEY_DATA_DIR_LOCATION" "${INSTALLER_YAML}" "false"
    productHome="$YAML_VALUE"
    if [ -z "${productHome}" ]; then
        return
    elif [ "$WRAPPER_SCRIPT_TYPE" == "$WRAPPER_SCRIPT_TYPE_DOCKER_COMPOSE" ]; then
        productHome="$productHome/var"
    fi
    echo -n "$productHome"
}

wrapper_hook_getUserInputs() {
    logSilly "Method ${FUNCNAME[0]} (unimplemented)"
}

wrapper_hook_askExtraPostgresDetails(){
    logSilly "Method ${FUNCNAME[0]} (unimplemented)"
}

reconfirmPostgresPassword(){
    unset USER_CHOICE
    while [ -z "$USER_CHOICE" ]; do
        echo -n "confirm $PROMPT_DB_PASSWORD: "
        read -s USER_CHOICE
        echo ""
        if [ -z "$USER_CHOICE" ]; then
            warn "Invalid Entry"
        fi
    done
    wrapper_setSystemValue "$SYS_KEY_SHARED_DATABASE_PASSWORD" "$USER_CHOICE" "$FLAG_Y"
    getSystemValue "$SYS_KEY_SHARED_DATABASE_PASSWORD" "NOT_SET" "false" "$INSTALLER_YAML"
    RPM_DEB_POSTGRES_PASSWORD=${YAML_VALUE}
}

wrapper_askPostgresPassword() {
    _updateFromSystemYaml "SHARED_DATABASE_PASSWORD" "KEY_DB_PASSWORD"
    if [ -z "$YAML_VALUE" ]; then
        wrapper_askUser "DB_PASSWORD"
        getSystemValue "$SYS_KEY_SHARED_DATABASE_PASSWORD" "NOT_SET" "false" "$INSTALLER_YAML"
        RPM_DEB_POSTGRES_FIRST_PASSWORD=${YAML_VALUE}
        reconfirmPostgresPassword
        if [[ "${RPM_DEB_POSTGRES_FIRST_PASSWORD}" == "${RPM_DEB_POSTGRES_PASSWORD}" ]]; then
            return
        fi
        warn "Confirmed $PROMPT_DB_PASSWORD doesn't match"
        wrapper_askPostgresPassword
    else
        logger "Found $PROMPT_DB_PASSWORD in system.yaml. Skipping prompt"
    fi
}

wrapper_getHostIP(){
    if [[ ! -z "${SYS_KEY_SHARED_NODE_IP}" && ! -z "${INSTALLER_YAML}" && -f "${INSTALLER_YAML}" ]]; then
        getSystemValue "$SYS_KEY_SHARED_NODE_IP" "NOT_SET" "false" "$INSTALLER_YAML"
        if [[ ! -z "${YAML_VALUE}" && "${YAML_VALUE}" != "NOT_SET" ]]; then
            echo -n "${YAML_VALUE}"
            return 0;
        fi
    fi

    echo -n "$(io_getPublicHostIP)"
}

getJFrogURLIfAvailable() {
    if [ ! -z "$JF_SHARED_JFROGURL" ]; then
        return
    fi
    logDebug "Method ${FUNCNAME[0]} Caller:$(caller)"
    local productHome="$(_getDataDir)"
    if [ -z "${productHome}" ]; then
        logDebug "No key: [$KEY_DATA_DIR_LOCATION] found. Aborting jfrog URL search"
        return
    fi
    
    local systemYaml="${productHome}/etc/${FILE_NAME_SYSTEM_YAML}"
    getYamlValue "$SYS_KEY_SHARED_JFROGURL" "${systemYaml}" "false"
    if [ ! -z "${YAML_VALUE}" ]; then
        logger "$PROMPT_JFROGURL: [$SYS_KEY_SHARED_JFROGURL] found in system.yaml. Skipping prompt"
        JF_SHARED_JFROGURL="$YAML_VALUE"
        return
    fi
    logDebug "JFrog URL not found"
}

_getDatabaseType() {
    logDebug "Method ${FUNCNAME[0]} Caller:$(caller)"
    
    DATABASE_TYPE=
    local productHome=
    if [[ "${TRIAL_FLOW}" == "${FLAG_Y}" ]]; then
        productHome="$(_getRTDataDir)"
    else
        productHome="$(_getDataDir)"
    fi
    if [ -z "${productHome}" ]; then
        logDebug "No key: [$KEY_DATA_DIR_LOCATION] found. Aborting database type search"
        return
    fi
    
    local systemYaml="${productHome}/etc/${FILE_NAME_SYSTEM_YAML}"

    # Get the db type configured
    getYamlValue "$SYS_KEY_SHARED_DATABASE_TYPE" "$systemYaml" "false"
    DATABASE_TYPE="$YAML_VALUE"
}

_getJoinKeyIfAvailable() {
    if [ ! -z "$JF_SHARED_SECURITY_JOINKEY" ]; then
        return
    fi
    logDebug "Method ${FUNCNAME[0]} Caller:$(caller)"
    local productHome="$(_getDataDir)"
    if [ -z "${productHome}" ]; then
        logDebug "No key: [$KEY_DATA_DIR_LOCATION] found. Aborting join key search"
        return
    fi
    
    # First check if a file exists
    local joinKeyFile="${productHome}/etc/security/${FILE_NAME_JOIN_KEY}"
    if [ -f "$joinKeyFile" ];then
        logger "$PROMPT_JOIN_KEY file: [$joinKeyFile] found. Skipping prompt"
        JF_SHARED_SECURITY_JOINKEY=$(cat "$joinKeyFile")
        return
    fi
    
    local systemYaml="${productHome}/etc/${FILE_NAME_SYSTEM_YAML}"
    if [ ! -f "$systemYaml" ]; then
        return
    fi
    
    getYamlValue "$SYS_KEY_SHARED_SECURITY_JOINKEY" "${systemYaml}" "false"
    if [ ! -z "${YAML_VALUE}" ]; then
        logger "$PROMPT_JOIN_KEY: [$SYS_KEY_SHARED_SECURITY_JOINKEY] found in system.yaml. Skipping prompt"
        JF_SHARED_SECURITY_JOINKEY="$YAML_VALUE"
        return
    fi
    logDebug "Join key not found"
}

_getHostIpIfAvailable() {
    if [ ! -z "$JF_SHARED_NODE_IP" ]; then
        return
    fi
    logDebug "Method ${FUNCNAME[0]} Caller:$(caller)"
    local productHome="$(_getDataDir)"
    if [ -z "${productHome}" ]; then
        logDebug "No key: [$KEY_DATA_DIR_LOCATION] found. Aborting Host IP search"
        return
    fi
    
    local systemYaml="${productHome}/etc/${FILE_NAME_SYSTEM_YAML}"
    if [ ! -f "$systemYaml" ]; then
        return
    fi
    
    getYamlValue "$SYS_KEY_SHARED_NODE_IP" "${systemYaml}" "false"
    if [ ! -z "${YAML_VALUE}" ]; then
        logger "$PROMPT_HOST_IP: [$SYS_KEY_SHARED_NODE_IP] found in system.yaml. Skipping prompt"
        JF_SHARED_NODE_IP="$YAML_VALUE"
        return
    fi
    logDebug "Host IP not found"
}

_getMasterKeyIfAvailable() {
    if [ ! -z "$JF_SHARED_SECURITY_MASTERKEY" ]; then
        return
    fi
    logDebug "Method ${FUNCNAME[0]} Caller:$(caller)"
    local productHome="$(_getDataDir)"
    if [ -z "${productHome}" ]; then
        logDebug "No key: [$KEY_DATA_DIR_LOCATION] found. Aborting join key search"
        return
    fi
    
    # First check if a file exists
    local masterKeyFile="${productHome}/etc/security/${FILE_NAME_MASTER_KEY}"
    if [ -f "$masterKeyFile" ];then
        logger "$PROMPT_CLUSTER_MASTER_KEY file: [$masterKeyFile] found. Skipping prompt"
        JF_SHARED_SECURITY_MASTERKEY=$(cat "$masterKeyFile")
        return
    fi
    
    local systemYaml="${productHome}/etc/${FILE_NAME_SYSTEM_YAML}"
    if [ ! -f "$systemYaml" ]; then
        return
    fi
    
    getYamlValue "$SYS_KEY_SHARED_SECURITY_MASTERKEY" "${systemYaml}" "false"
    if [ ! -z "${YAML_VALUE}" ]; then
        logger "$PROMPT_CLUSTER_MASTER_KEY: [$SYS_KEY_SHARED_SECURITY_MASTERKEY] found in system.yaml [$systemYaml]. Skipping prompt"
        JF_SHARED_SECURITY_MASTERKEY="$YAML_VALUE"
        return
    fi
    logDebug "Master key not found"
}

_askForMandatoryInputs() {
    logDebug "Method ${FUNCNAME[0]} Caller:$(caller)"
    local mandatoryFields=($MANDATORY_FIELDS)
    for key in "${mandatoryFields[@]}"; do
        case ${key} in
            JF_SHARED_JFROGURL)
                getJFrogURLIfAvailable
                if [ ! -z "$JF_SHARED_JFROGURL" ]; then
                    wrapper_setSystemValue "$KEY_JFROGURL" "$JF_SHARED_JFROGURL"
                else 
                    wrapper_askUser "JFROGURL"
                fi
            ;;
            JF_SHARED_SECURITY_JOINKEY)
                _getJoinKeyIfAvailable
                if [ ! -z "$JF_SHARED_SECURITY_JOINKEY" ]; then
                    wrapper_setSystemValue "$KEY_JOIN_KEY" "$JF_SHARED_SECURITY_JOINKEY"
                else
                    wrapper_askUser "JOIN_KEY"
                fi
            ;;
            JF_SHARED_SECURITY_MASTERKEY)
                _getMasterKeyIfAvailable
                if [ ! -z "$JF_SHARED_SECURITY_MASTERKEY" ]; then
                    wrapper_setSystemValue "$KEY_CLUSTER_MASTER_KEY" "$JF_SHARED_SECURITY_MASTERKEY"
                else
                    wrapper_askUser "CLUSTER_MASTER_KEY"
                fi
            ;;
        esac
    done
}

_updateFromSystemYaml() {
    logDebug "Method ${FUNCNAME[0]} Caller:$(caller)"  
    local keyToFetch=$1
    local installerYamlKey=$2
    local customYaml="$3"
    unset YAML_VALUE

    if [ ! -f "$INSTALLER_YAML" ]; then
        logDebug "No installer yaml found. Skipping step"
        return
    fi

    local productHome="$(_getDataDir)"
    if [ -z "${productHome}" ]; then
        logDebug "No key: [$KEY_DATA_DIR_LOCATION] found. Aborting jfrog URL search"
        return
    fi

    local systemYaml=${customYaml:-"${productHome}/etc/${FILE_NAME_SYSTEM_YAML}"}
    if [ ! -f "$systemYaml" ]; then
        return
    fi

    eval sysYamlKey=\${SYS_KEY_${keyToFetch}}
    if [ -z "$installerYamlKey" ]; then
        eval installerYamlKey=\${KEY_${keyToFetch}}
    fi
    eval envVariable=\${JF_${keyToFetch}}

    getYamlValue "$sysYamlKey" "$systemYaml" "false"
    if [ ! -z "$YAML_VALUE" ]; then
        local tempValue="$YAML_VALUE"
        logDebug "Overiding installerYamlKey [$installerYamlKey] with value of sysYamlKey [$sysYamlKey]: [$YAML_VALUE]"
        wrapper_setSystemValue "${installerYamlKey}" "$YAML_VALUE" "$FLAG_Y"
        YAML_VALUE="$tempValue"
        return
    fi
    logDebug "No value found in system.yaml for [$keyToFetch]. sysYamlKey [$sysYamlKey]. Not overriding"
    unset YAML_VALUE
}

# This can be used to decide if a prompt needs to be displayed
# This will return 0 if provided key is already set in source yaml
_skipPrompt(){
    local srcKey="$1"
    local targetKey="$2"
    local srcYaml="$3"

    if [[ -z "${srcKey}" || -z "${srcYaml}" ]]; then
        return 1;
    fi

    _updateFromSystemYaml "$srcKey" "$targetKey" "${srcYaml}"
    if [ ! -z "$YAML_VALUE" ]; then
        logger "Found flag ${srcKey} as [${YAML_VALUE}]. Skipping prompt"
        return 0;
    else
        return 1;
    fi
}

_setDatabaseDriver(){
    local databaseType="$1"
    local databaseDriver=""
    
    if [ -z "${databaseType}" ]; then
        getSystemValue "$KEY_DATABASE_TYPE" "NOT_SET" "false" "$INSTALLER_YAML"
        [ "${YAML_VALUE}" != "NOT_SET" ] && databaseType="${YAML_VALUE}" || true
    fi

    databaseDriver=$(getDatabaseDriver "${databaseType}")
    if [ ! -z "${databaseDriver}" ]; then
        setSystemValue "${SYS_KEY_SHARED_DATABASE_DRIVER}" "$databaseDriver" "$INSTALLER_YAML"
    fi
}

_askExternalDBInputs(){
    _updateFromSystemYaml "SHARED_DATABASE_URL" "$KEY_DATABASE_URL"
    if [ ! -z "$YAML_VALUE" ]; then
        logger "Found URL [$YAML_VALUE] in system.yaml. Skipping prompt"
        return 0
    fi

    if [[ "$FLAG_MULTIPLE_DB_SUPPORT" == "$FLAG_Y" ]]; then
        _updateFromSystemYaml "SHARED_DATABASE_TYPE" "KEY_DATABASE_TYPE"
        local databaseType="$YAML_VALUE"
        [ -z "$databaseType" ] && wrapper_askUser "DATABASE_TYPE" || logger "Found $PROMPT_DATABASE_TYPE in system.yaml. Skipping prompt"
        
        if [ -z "$databaseType" ]; then
            getSystemValue "$KEY_DATABASE_TYPE" "NOT_SET" "false" "$INSTALLER_YAML"
            [ ! -z "$YAML_VALUE" ] && databaseType="${YAML_VALUE}" || true
        fi
        
        if [ "${databaseType}" == "${SYS_KEY_SHARED_DATABASE_TYPE_VALUE_DERBY}" ]; then
            logger "Database type set as [${databaseType}]. Skipping database related prompts"
            return 0
        fi

        _setDatabaseDriver "${databaseType}"
    else
        setSystemValue "${SYS_KEY_SHARED_DATABASE_TYPE}" "$SYS_KEY_SHARED_DATABASE_TYPE_VALUE_POSTGRES" "$INSTALLER_YAML"
        _setDatabaseDriver "${SYS_KEY_SHARED_DATABASE_TYPE_VALUE_POSTGRES}"
    fi

    wrapper_askUser "DATABASE_URL"
    if [ "$FLAG_MULTIPLE_PG_SCHEME" == "$FLAG_Y" ]; then
        logBold "$MESSAGE_MULTIPLE_PG_SCHEME"
        _updateFromSystemYaml "SHARED_DATABASE_PASSWORD" "KEY_DATABASE_PASSWORD"
        [ -z "$YAML_VALUE" ] && wrapper_askUser "DATABASE_PASSWORD" || logger "Found $PROMPT_DATABASE_PASSWORD in system.yaml. Skipping prompt"
    else
        _updateFromSystemYaml "SHARED_DATABASE_USERNAME" "KEY_DATABASE_USERNAME"
        [ -z "$YAML_VALUE" ] && wrapper_askUser "DATABASE_USERNAME" || logger "Found $PROMPT_DATABASE_USERNAME in system.yaml. Skipping prompt"
        _updateFromSystemYaml "SHARED_DATABASE_PASSWORD" "KEY_DATABASE_PASSWORD"
        [ -z "$YAML_VALUE" ] && wrapper_askUser "DATABASE_PASSWORD" || logger "Found $PROMPT_DATABASE_PASSWORD in system.yaml. Skipping prompt"
    fi
}

_askPostgresDetails() {
    logDebug "Method ${FUNCNAME[0]} Caller:$(caller)"
    local isPostgresExternalised="$FLAG_Y"
    local productHome="$(_getDataDir)"
    local systemYaml="${productHome}/etc/${FILE_NAME_SYSTEM_YAML}"

    if [[ ! -z "${SKIP_POSTGRES_SETUP}" && "${SKIP_POSTGRES_SETUP}" == "${FLAG_Y}" ]]; then
        return 0;
    fi
    
    #For Backward compatibility
    JF_INSTALL_POSTGRES=${JF_INSTALL_POSTGRES:-"$INSTALL_POSTGRES"}
    if [ "$JF_NODE_TYPE" == "$NODE_TYPE_STANDALONE" ]; then
        if [ ! -z "$JF_INSTALL_POSTGRES" ]; then
            JF_INSTALL_POSTGRES="$(echo "$JF_INSTALL_POSTGRES" | tr '[:upper:]' '[:lower:]')"
            logger "Found flag INSTALL_POSTGRES. Skipping prompt"
            wrapper_setSystemValue "$KEY_POSTGRES_INSTALL" "$JF_INSTALL_POSTGRES" "${FLAG_Y}"
        else
            # # Commented - will not use bundled postgresql.
            logDebug "Bundled postgresql not available"
            # _skipPrompt "POSTGRES_INSTALL" "" "${INSTALLER_STATE_YAML_FILE}" \
            #     || wrapper_askUser "POSTGRES_INSTALL"
        fi
        getYamlValue "$KEY_POSTGRES_INSTALL" "$INSTALLER_YAML" "false"
        isPostgresExternalised=$([ "$YAML_VALUE" == "$FLAG_Y" ] && echo -n "$FLAG_N" || echo -n "$FLAG_Y")
        getYamlValue "$SYS_KEY_SHARED_DATABASE_URL" "${systemYaml}" "false"
        isDataBaseUrlExist="${YAML_VALUE}"
        if [[ "$isPostgresExternalised" == "$FLAG_N" && -z "${isDataBaseUrlExist}" ]]; then
            setSystemValue "${SYS_KEY_SHARED_DATABASE_TYPE}" "$SYS_KEY_SHARED_DATABASE_TYPE_VALUE_POSTGRES" "$INSTALLER_YAML"
            _setDatabaseDriver "${SYS_KEY_SHARED_DATABASE_TYPE_VALUE_POSTGRES}"
            wrapper_askPostgresPassword
            wrapper_hook_askExtraPostgresDetails
        fi
    fi

    if [ "$isPostgresExternalised" == "$FLAG_Y" ]; then
        _askExternalDBInputs
    fi
}

wrapper_setPostgresVersion() {
    local productHome=
    productHome="$(_getDataDir)"
    local systemYaml="${productHome}/etc/${FILE_NAME_SYSTEM_YAML}"
    local postgresVer=${JFROG_POSTGRES_15_6X_VERSION}
    if [[ "${TRIAL_FLOW}" == "${FLAG_Y}" ]]; then
        _updateFromSystemYaml "POSTGRES_VERSION" "" "${INSTALLER_STATE_YAML_FILE_RT}"
    else
        _updateFromSystemYaml "POSTGRES_VERSION" "" "${INSTALLER_STATE_YAML_FILE}"
    fi
    local installedPostgresVer=${YAML_VALUE}
    
    if [[ "${PRODUCT_NAME}" == "$ARTIFACTORY_LABEL" ]]; then
        _getDatabaseType
        if [[ ! -z "$DATABASE_TYPE" && "$DATABASE_TYPE" != "$SYS_KEY_SHARED_DATABASE_TYPE_VALUE_POSTGRES" ]]; then
            return
        fi
    fi
    if [[ -z "${installedPostgresVer}" || "${installedPostgresVer}" == "NOT_SET" ]]; then
        getYamlValue "$SYS_KEY_SHARED_DATABASE_URL" "${systemYaml}" "false"
        local isDataBaseUrlExist="${YAML_VALUE}"
        getYamlValue "$KEY_POSTGRES_INSTALL" "$INSTALLER_YAML" "false"
        local isPostgresExternalised=$([ "$YAML_VALUE" == "$FLAG_Y" ] && echo -n "$FLAG_N" || echo -n "$FLAG_Y")
        if [ "$isPostgresExternalised" == "$FLAG_Y" ]; then 
            return
        fi

        if [ "$isPostgresExternalised" == "$FLAG_N" ] && [ ! -z "${isDataBaseUrlExist}" ]; then
            postgresVer=${JFROG_POSTGRES_9X_VERSION}
        fi
        wrapper_setSystemValue "$KEY_POSTGRES_VERSION" "${postgresVer}" "${overwrite}"
    fi
    
    if [ "$isPostgresExternalised" == "$FLAG_N" ] && [ ! -z "${isDataBaseUrlExist}" ]; then
        case "$installedPostgresVer" in
            $JFROG_POSTGRES_15_2X_VERSION)
                postgresVer=${JFROG_POSTGRES_15_2X_VERSION}
            ;;
        esac
        wrapper_setSystemValue "$KEY_POSTGRES_VERSION" "${postgresVer}" "${overwrite}"
    fi
}

_removeEmptyLines(){
    SEDOPTION="-i ''"
    if [[ $(uname) != "Darwin" ]]; then
        SEDOPTION="-i"
    fi
    sed $SEDOPTION -n -e '/^*$/d' -e '/^[[:space:]]*$/d' -e '/^\s*$/d' -e '/^$/d' -e '/^\s*$/!p' "$1"
}

_askForDBInputs() {
    logDebug "Method ${FUNCNAME[0]} Caller:$(caller)"
    local databases=($EXTERNAL_DATABASES)
    for key in "${databases[@]}"; do
        case ${key} in
            $DATABASE_POSTGRES)
                _getDatabaseType
                if [ "$FLAG_MULTIPLE_DB_SUPPORT" == "$FLAG_Y" ]; then
                    # If multiple database is support by the product ask for database type and questions associated for that type
                    _askPostgresDetails
                elif [ ! -z "$DATABASE_TYPE" ] && [ "$DATABASE_TYPE" != "$SYS_KEY_SHARED_DATABASE_TYPE_VALUE_POSTGRES" ]; then
                    # If the database is not postgres, skip all the associated prompts.
                    logger "Found [$SYS_KEY_SHARED_DATABASE_TYPE] as [$DATABASE_TYPE] in system.yaml. Skipping $POSTGRES_LABEL prompts"
                else
                    _askPostgresDetails
                fi
            ;;
        esac
    done
}

wrapper_getUserInputs() {
    logDebug "Method ${FUNCNAME[0]} Caller:$(caller)"

    wrapper_createInstallerYaml
    wrapper_setInstallerDefaults
    wrapper_hook_start_getUserInputs
    _askForMandatoryInputs
    _getHostIpIfAvailable
    if [ ! -z "$JF_SHARED_NODE_IP" ]; then
        wrapper_setSystemValue "$KEY_HOST_IP" "$JF_SHARED_NODE_IP"
    else 
        wrapper_askUser "HOST_IP"
    fi

    if [ "$DONT_PROMPT_USE_DEFAULTS" == "$FLAG_Y" ] || [ "$IS_UPGRADE" == "$FLAG_Y" ]; then
        return;        
    fi
    _askForDBInputs
    wrapper_hook_getUserInputs
}

# This method should NOT have logs since it returns a string
_isAValidChoice() {
    local userChoice="$1"
    if [ -z "$2" ]; then
        echo -n "$FLAG_Y"
    fi
    local validChoices=($2)
    for key in "${validChoices[@]}"; do
        if [ "$key" == "$userChoice" ]; then
            echo -n "$FLAG_Y"
        fi
    done
    echo -n "$FLAG_N"
}

## Utility method to get user inputs during installations
## Parameters: 
##  message (will be displayed to the user)
##  yamlKey (will be used to read the default value)
##  yamlFile (will be used to read the default value)
##  validValues (if passed, the user will be forced to choose from these inputs)
## Globals:
## DONT_PROMPT_USE_DEFAULTS: if set to "yes", the user will not be prompted and the default value will be set
## USER_CHOICE: will contain the user's choice
## Returns: none
sys_askUserYaml(){
    local message="$1"
    local yamlKey="$2"
    local yamlFile="$3"
    local validValuesAsString="$4"

    # validate that this method has everything it needs to proceed
    if [ -z "${message}" ] || [ -z "${yamlKey}" ] || [ -z "${yamlFile}" ] || [ ! -f "${yamlFile}" ]; then
        warn "Invalid invocation of method."
        return 1
    fi

    # read the yaml file
    getYamlValue "${yamlKey}" "${yamlFile}" "false"
    local defaultValue="${YAML_VALUE}"

    sys_askUser "$message" "$defaultValue" "$validValuesAsString"

    if [ "$USER_CHOICE" != "$FLAG_NOT_APPLICABLE" ]; then
        setSystemValue "$yamlKey" "$USER_CHOICE" "$yamlFile"
    fi
}

## Utility method to get user inputs during installations
## Parameters: 
##  yamlKey (will be used to read the default value)
## Globals:
## DONT_PROMPT_USE_DEFAULTS: if set to "yes", the user will not be prompted and the default value will be set
## USER_CHOICE: will contain the user's choice
## Returns: none
wrapper_askUser(){
    logDebug "Method ${FUNCNAME[0]} Caller:$(caller)"
    local yamlKey="$1"
    local yamlFile="$INSTALLER_YAML"

    # validate that this method has everything it needs to proceed
    if [ -z "${yamlKey}" ]; then
        warn "Invalid invocation of method."
        return 1
    fi

    eval key=\${KEY_${yamlKey}}
    if [ -z "$key" ]; then
        warn "Invalid invocation of method."
        return 1
    fi

    # read the yaml file
    getYamlValue "${key}" "${yamlFile}" "$FLAG_N"
    local defaultValue="${YAML_VALUE}"

    _getMethodOutputOrVariableValue "MESSAGE_${yamlKey}"
    if [ ! -z "$EFFECTIVE_MESSAGE" ]; then
        _messageBeforePrompt "$EFFECTIVE_MESSAGE"
    else
        #If there is no message before a prompt, add a new line explicitly. The message takes care of it usually
        echo ""
    fi
    
    _getMethodOutputOrVariableValue "PROMPT_${yamlKey}"
    local prompt="$EFFECTIVE_MESSAGE"

    if [ -z "$prompt" ]; then
        warn "Invalid key. No prompt found"
        return 1
    fi
    sys_askUser "$prompt" "$defaultValue" "$yamlKey"
    
    modifyValue "USER_CHOICE" "${yamlKey}"

    if [ "$USER_CHOICE" != "$FLAG_NOT_APPLICABLE" ]; then
        setSystemValue "${key}" "$USER_CHOICE" "$yamlFile"
    fi
}

_getPromptWithDefaults() {
    local validValuesAsString="$1"
    local defaulfValue="$2"

    if [ ! -z "$validValuesAsString" ] && [ -z "$defaultValue" ]; then
       echo -n " [$(echo $validValuesAsString | sed -e "s# #/#g" )]" # replaces spaces with /
    elif [ -z "$validValuesAsString" ] && [ ! -z "$defaultValue" ]; then
        echo -n " (Default: $defaultValue)"
    elif [ ! -z "$validValuesAsString" ]; then
        local validValues=($validValuesAsString)
        local textToDisplay=
        for key in "${validValues[@]}"; do
            if [ "$defaultValue" == "$key" ];then
                key="$(echo "$key" | tr '[:lower:]' '[:upper:]')"
            fi
            textToDisplay="$textToDisplay $key"
        done
        echo -n " [$(echo $textToDisplay | sed -e "s# #/#g" )]" # replaces spaces with /
    fi
    
}

## Utility method to get user inputs
## Parameters: 
##  message (will be displayed to the user)
##  defaultValue (will be the default value selectable by pressing enter)
##  validValues (if passed, the user will be forced to choose from these inputs)
## Globals:
## DONT_PROMPT_USE_DEFAULTS: if set to "yes", the user will not be prompted and the default value will be set
## USER_CHOICE: will contain the user's choice
## Returns: none
sys_askUser(){
    local message="$1"
    local defaultValue="$2"
    local yamlKey="$3"

    _getMethodOutputOrVariableValue "VALID_VALUES_${yamlKey}"
    local validValuesAsString="$EFFECTIVE_MESSAGE"
    
    _getMethodOutputOrVariableValue "REGEX_${yamlKey}"
    local regexToValidate="$EFFECTIVE_MESSAGE"
    
    _getMethodOutputOrVariableValue "ERROR_MESSAGE_${yamlKey}"
    local regexValidationMessage="$EFFECTIVE_MESSAGE"

    # validate that this method has everything it needs to proceed
    if [ -z "${message}" ]; then
        warn "Invalid invocation of method."
        return 1
    fi

    eval isASensitiveKey=\${IS_SENSITIVE_${yamlKey}}
    eval isOptional=\${IS_OPTIONAL_${yamlKey}}

    if [ ! -z "$defaultValue" ]; then
        if [ "${DONT_PROMPT_USE_DEFAULTS}" == "$FLAG_Y" ]; then
            # If the global flag is set to never prompt the user, just use the default value and return
            USER_CHOICE="${defaultValue}"
            return 0
        fi
        if [ ! -z "$isASensitiveKey" ]; then
            message="$message (****)"
        fi
    fi
    if [ -z "$isASensitiveKey" ]; then
        message="$message$(_getPromptWithDefaults "$validValuesAsString" "$defaultValue")"
    fi

    # If not, prompt the user
    echo -n "$message: "
    USER_CHOICE=
    while [ -z "$USER_CHOICE" ]; do
        if [ ! -z "$isASensitiveKey" ]; then
            read -s USER_CHOICE
            echo "" # new line after senstitive key is entered
        else
            read USER_CHOICE
        fi

        : ${USER_CHOICE:="$defaultValue"}
        # IF the value is a flag, lower case it
        if [ ! -z "$validValuesAsString" ]; then
            USER_CHOICE="$(echo "$USER_CHOICE" | tr '[:upper:]' '[:lower:]')"
            if [ $(_isAValidChoice "$USER_CHOICE" "$validValuesAsString") == "$FLAG_N" ]; then
                warn "Invalid choice. Please retry"
                echo -n "$message: "
                unset USER_CHOICE
            fi
        elif [ ! -z "$regexToValidate" ]; then
                validateRegex "$USER_CHOICE" "$regexToValidate" || {
                logDebug "The input failed validations [$regexToValidate]"
                warn "$regexValidationMessage"
                echo -n "$message: "
                unset USER_CHOICE
            }
        elif [ -z "$USER_CHOICE" ] && [ "$isOptional" == "$FLAG_Y" ]; then
            USER_CHOICE="$FLAG_NOT_APPLICABLE"
        fi
    done
    if [ ! -z "$isASensitiveKey" ]; then
        echo "" # new line after senstitive key is entered
    fi
}

askPostgresUpgrade() {
    if [[ "${SKIP_POSTGRES_SETUP}" == "${FLAG_Y}" ]]; then
        warn "Proceeding with $PRODUCT_NAME upgrade with $POSTGRES_LABEL ${POSTGRES_OLD}"
        return 0;
    fi
    warn "$PRODUCT_NAME is currently using $POSTGRES_LABEL ${POSTGRES_OLD}"
    echo -e "It is highly recommended to upgrade your $POSTGRES_LABEL DB."
    echo -e "The Docker Compose template with $POSTGRES_LABEL ${POSTGRES_VERSION}v is available here [$COMPOSE_TEMPLATES/docker-compose-postgres.yaml]"
    echo -e "For more information on $POSTGRES_LABEL upgrades, see https://www.postgresql.org/docs/${POSTGRES_MAJOR_VER}/upgrading.html"
    echo -e "Continuing will upgrade $PRODUCT_NAME with the existing $POSTGRES_LABEL ${POSTGRES_OLD}."
    echo -n "Do you wish to exit the upgrade script and to upgrade your $POSTGRES_LABEL to ${POSTGRES_VERSION}v? [Y/n]: "
    read USER_CHOICE
    echo ""
    USER_CHOICE="$(echo "$USER_CHOICE" | tr '[:upper:]' '[:lower:]')"
    while [[ -z $USER_CHOICE || ! ${USER_CHOICE} == $FLAG_N && ! ${USER_CHOICE} == $FLAG_Y ]]; do
        warn "Invalid choice. Please retry"
        echo -n "Do you want to abort the upgrade script and upgrade your $POSTGRES_LABEL to a ${POSTGRES_VERSION}v? [Y/n]: "
        read USER_CHOICE
        USER_CHOICE="$(echo "$USER_CHOICE" | tr '[:upper:]' '[:lower:]')"
    done
    if [[ "${USER_CHOICE}" == "${FLAG_N}" ]]; then
        unset USER_CHOICE
        warn "Proceeding with $PRODUCT_NAME upgrade with $POSTGRES_LABEL ${POSTGRES_OLD}"
    fi
    if [[ "${USER_CHOICE}" == "${FLAG_Y}" ]]; then
        unset USER_CHOICE
        exit 0
    fi
}
